@@ -30,7 +30,7 @@
 -record(state, {connection,
                 channel_sup_sup,
                 map_num_pa      = gb_trees:empty(), %% Number -> {Pid, AState}
-                map_pid_num     = dict:new(),       %% Pid -> Number
+                map_pid_num     = #{},       %% Pid -> Number
                 channel_max     = ?MAX_CHANNEL_NUMBER,
                 closing         = false}).
 
@@ -43,16 +43,16 @@ start_link(Connection, ConnName, ChSupSup) ->
 
 open_channel(ChMgr, ProposedNumber, Consumer, InfraArgs) ->
     gen_server:call(ChMgr, {open_channel, ProposedNumber, Consumer, InfraArgs},
-                     infinity).
+                     amqp_util:call_timeout()).
 
 set_channel_max(ChMgr, ChannelMax) ->
     gen_server:cast(ChMgr, {set_channel_max, ChannelMax}).
 
 is_empty(ChMgr) ->
-    gen_server:call(ChMgr, is_empty, infinity).
+    gen_server:call(ChMgr, is_empty, amqp_util:call_timeout()).
 
 num_channels(ChMgr) ->
-    gen_server:call(ChMgr, num_channels, infinity).
+    gen_server:call(ChMgr, num_channels, amqp_util:call_timeout()).
 
 pass_frame(ChMgr, ChNumber, Frame) ->
     gen_server:cast(ChMgr, {pass_frame, ChNumber, Frame}).
@@ -63,10 +63,10 @@ signal_connection_closing(ChMgr, ChannelCloseType, Reason) ->
 process_channel_frame(Frame, Channel, ChPid, AState) ->
     case rabbit_command_assembler:process(Frame, AState) of
         {ok, NewAState}                  -> NewAState;
-        {ok, Method, NewAState}          -> rabbit_channel:do(ChPid, Method),
+        {ok, Method, NewAState}          -> rabbit_channel_common:do(ChPid, Method),
                                             NewAState;
-        {ok, Method, Content, NewAState} -> rabbit_channel:do(ChPid, Method,
-                                                              Content),
+        {ok, Method, Content, NewAState} -> rabbit_channel_common:do(ChPid, Method,
+                                                                     Content),
                                             NewAState;
         {error, Reason}                  -> ChPid ! {channel_exit, Channel,
                                                      Reason},
@@ -222,14 +222,14 @@ internal_pass_frame(Number, Frame, State) ->
 internal_register(Number, Pid, AState,
                   State = #state{map_num_pa = MapNPA, map_pid_num = MapPN}) ->
     MapNPA1 = gb_trees:enter(Number, {Pid, AState}, MapNPA),
-    MapPN1 = dict:store(Pid, Number, MapPN),
+    MapPN1 = maps:put(Pid, Number, MapPN),
     State#state{map_num_pa  = MapNPA1,
                 map_pid_num = MapPN1}.
 
 internal_unregister(Number, Pid,
                     State = #state{map_num_pa = MapNPA, map_pid_num = MapPN}) ->
     MapNPA1 = gb_trees:delete(Number, MapNPA),
-    MapPN1 = dict:erase(Pid, MapPN),
+    MapPN1 = maps:remove(Pid, MapPN),
     State#state{map_num_pa  = MapNPA1,
                 map_pid_num = MapPN1}.
 
@@ -245,7 +245,7 @@ internal_lookup_npa(Number, #state{map_num_pa = MapNPA}) ->
     end.
 
 internal_lookup_pn(Pid, #state{map_pid_num = MapPN}) ->
-    case dict:find(Pid, MapPN) of {ok, Number} -> Number;
+    case maps:find(Pid, MapPN) of {ok, Number} -> Number;
                                   error        -> undefined
     end.
 
@@ -255,4 +255,4 @@ internal_update_npa(Number, Pid, AState, State = #state{map_num_pa = MapNPA}) ->
 signal_channels_connection_closing(ChannelCloseType, Reason,
                                    #state{map_pid_num = MapPN}) ->
     [amqp_channel:connection_closing(Pid, ChannelCloseType, Reason)
-        || Pid <- dict:fetch_keys(MapPN)].
+        || Pid <- maps:keys(MapPN)].
