@@ -44,4 +44,4 @@ start_connection_sup(AmqpParams) ->
 init([]) ->
     {ok, {{simple_one_for_one, 0, 1},
           [{connection_sup, {amqp_connection_sup, start_link, []},
-           temporary, infinity, supervisor, [amqp_connection_sup]}]}}.
+           temporary, ?SUPERVISOR_WAIT, supervisor, [amqp_connection_sup]}]}}.
