@@ -1,4 +1,4 @@
-%% Copyright (c) 2013-2014, Loïc Hoguin <essen@ninenines.eu>
+%% Copyright (c) 2013-2016, Loïc Hoguin <essen@ninenines.eu>
 %%
 %% Permission to use, copy, modify, and/or distribute this software for any
 %% purpose with or without fee is hereby granted, provided that the above
@@ -444,6 +444,7 @@ all_ext(<<"m2v">>) -> {<<"video">>, <<"mpeg">>, []};
 all_ext(<<"m3a">>) -> {<<"audio">>, <<"mpeg">>, []};
 all_ext(<<"m3u8">>) -> {<<"application">>, <<"vnd.apple.mpegurl">>, []};
 all_ext(<<"m3u">>) -> {<<"audio">>, <<"x-mpegurl">>, []};
+all_ext(<<"m4a">>) -> {<<"audio">>, <<"mp4">>, []};
 all_ext(<<"m4u">>) -> {<<"video">>, <<"vnd.mpegurl">>, []};
 all_ext(<<"m4v">>) -> {<<"video">>, <<"x-m4v">>, []};
 all_ext(<<"ma">>) -> {<<"application">>, <<"mathematica">>, []};
@@ -584,7 +585,7 @@ all_ext(<<"org">>) -> {<<"application">>, <<"vnd.lotus-organizer">>, []};
 all_ext(<<"osf">>) -> {<<"application">>, <<"vnd.yamaha.openscoreformat">>, []};
 all_ext(<<"osfpvg">>) -> {<<"application">>, <<"vnd.yamaha.openscoreformat.osfpvg+xml">>, []};
 all_ext(<<"otc">>) -> {<<"application">>, <<"vnd.oasis.opendocument.chart-template">>, []};
-all_ext(<<"otf">>) -> {<<"application">>, <<"x-font-otf">>, []};
+all_ext(<<"otf">>) -> {<<"font">>, <<"otf">>, []};
 all_ext(<<"otg">>) -> {<<"application">>, <<"vnd.oasis.opendocument.graphics-template">>, []};
 all_ext(<<"oth">>) -> {<<"application">>, <<"vnd.oasis.opendocument.text-web">>, []};
 all_ext(<<"oti">>) -> {<<"application">>, <<"vnd.oasis.opendocument.image-template">>, []};
@@ -836,9 +837,9 @@ all_ext(<<"trm">>) -> {<<"application">>, <<"x-msterminal">>, []};
 all_ext(<<"tr">>) -> {<<"text">>, <<"troff">>, []};
 all_ext(<<"tsd">>) -> {<<"application">>, <<"timestamped-data">>, []};
 all_ext(<<"tsv">>) -> {<<"text">>, <<"tab-separated-values">>, []};
-all_ext(<<"ttc">>) -> {<<"application">>, <<"x-font-ttf">>, []};
+all_ext(<<"ttc">>) -> {<<"font">>, <<"collection">>, []};
 all_ext(<<"t">>) -> {<<"text">>, <<"troff">>, []};
-all_ext(<<"ttf">>) -> {<<"application">>, <<"x-font-ttf">>, []};
+all_ext(<<"ttf">>) -> {<<"font">>, <<"ttf">>, []};
 all_ext(<<"ttl">>) -> {<<"text">>, <<"turtle">>, []};
 all_ext(<<"twd">>) -> {<<"application">>, <<"vnd.simtech-mindmapper">>, []};
 all_ext(<<"twds">>) -> {<<"application">>, <<"vnd.simtech-mindmapper">>, []};
@@ -933,7 +934,8 @@ all_ext(<<"wm">>) -> {<<"video">>, <<"x-ms-wm">>, []};
 all_ext(<<"wmv">>) -> {<<"video">>, <<"x-ms-wmv">>, []};
 all_ext(<<"wmx">>) -> {<<"video">>, <<"x-ms-wmx">>, []};
 all_ext(<<"wmz">>) -> {<<"application">>, <<"x-msmetafile">>, []};
-all_ext(<<"woff">>) -> {<<"application">>, <<"font-woff">>, []};
+all_ext(<<"woff2">>) -> {<<"font">>, <<"woff2">>, []};
+all_ext(<<"woff">>) -> {<<"font">>, <<"woff">>, []};
 all_ext(<<"wpd">>) -> {<<"application">>, <<"vnd.wordperfect">>, []};
 all_ext(<<"wpl">>) -> {<<"application">>, <<"vnd.ms-wpl">>, []};
 all_ext(<<"wps">>) -> {<<"application">>, <<"vnd.ms-works">>, []};
