@@ -16,15 +16,28 @@
 
 -module(rabbit).
 
+%% Transitional step until we can require Erlang/OTP 21 and
+%% use the now recommended try/catch syntax for obtaining the stack trace.
+-compile(nowarn_deprecated_function).
+
 -behaviour(application).
 
 -export([start/0, boot/0, stop/0,
-         stop_and_halt/0, await_startup/0, status/0, is_running/0,
-         is_running/1, environment/0, rotate_logs/1, force_event_refresh/1,
+         stop_and_halt/0, await_startup/0, await_startup/1,
+         status/0, is_running/0, alarms/0,
+         is_running/1, environment/0, rotate_logs/0, force_event_refresh/1,
          start_fhc/0]).
--export([start/2, stop/1]).
--export([start_apps/1, stop_apps/1]).
--export([log_location/1, config_files/0, decrypt_config/2]). %% for testing and mgmt-agent
+
+-export([start/2, stop/1, prep_stop/1]).
+-export([start_apps/1, start_apps/2, stop_apps/1]).
+-export([log_locations/0, config_files/0, decrypt_config/2]). %% for testing and mgmt-agent
+-export([is_booted/1]).
+
+-ifdef(TEST).
+
+-export([start_logger/0]).
+
+-endif.
 
 %%---------------------------------------------------------------------------
 %% Boot steps.
@@ -61,11 +74,17 @@
                     {requires,    database},
                     {enables,     external_infrastructure}]}).
 
+-rabbit_boot_step({code_server_cache,
+                   [{description, "code_server cache server"},
+                    {mfa,         {rabbit_sup, start_child, [code_server_cache]}},
+                    {requires,    rabbit_alarm},
+                    {enables,     file_handle_cache}]}).
+
 -rabbit_boot_step({file_handle_cache,
                    [{description, "file handle cache server"},
                     {mfa,         {rabbit, start_fhc, []}},
                     %% FHC needs memory monitor to be running
-                    {requires,    rabbit_alarm},
+                    {requires,    code_server_cache},
                     {enables,     worker_pool}]}).
 
 -rabbit_boot_step({worker_pool,
@@ -141,21 +160,23 @@
                    [{description, "core initialized"},
                     {requires,    kernel_ready}]}).
 
--rabbit_boot_step({empty_db_check,
-                   [{description, "empty DB check"},
-                    {mfa,         {?MODULE, maybe_insert_default_data, []}},
-                    {requires,    core_initialized},
-                    {enables,     routing_ready}]}).
+-rabbit_boot_step({upgrade_queues,
+                   [{description, "per-vhost message store migration"},
+                    {mfa,         {rabbit_upgrade,
+                                   maybe_migrate_queues_to_per_vhost_storage,
+                                   []}},
+                    {requires,    [core_initialized]},
+                    {enables,     recovery}]}).
 
 -rabbit_boot_step({recovery,
                    [{description, "exchange, queue and binding recovery"},
                     {mfa,         {rabbit, recover, []}},
-                    {requires,    core_initialized},
+                    {requires,    [core_initialized]},
                     {enables,     routing_ready}]}).
 
--rabbit_boot_step({mirrored_queues,
-                   [{description, "adding mirrors to queues"},
-                    {mfa,         {rabbit_mirror_queue_misc, on_node_up, []}},
+-rabbit_boot_step({empty_db_check,
+                   [{description, "empty DB check"},
+                    {mfa,         {?MODULE, maybe_insert_default_data, []}},
                     {requires,    recovery},
                     {enables,     routing_ready}]}).
 
@@ -163,25 +184,19 @@
                    [{description, "message delivery logic ready"},
                     {requires,    core_initialized}]}).
 
--rabbit_boot_step({log_relay,
-                   [{description, "error log relay"},
-                    {mfa,         {rabbit_sup, start_child,
-                                   [rabbit_error_logger_lifecycle,
-                                    supervised_lifecycle,
-                                    [rabbit_error_logger_lifecycle,
-                                     {rabbit_error_logger, start, []},
-                                     {rabbit_error_logger, stop,  []}]]}},
-                    {requires,    routing_ready},
-                    {enables,     networking}]}).
-
 -rabbit_boot_step({direct_client,
                    [{description, "direct client"},
                     {mfa,         {rabbit_direct, boot, []}},
-                    {requires,    log_relay}]}).
+                    {requires,    routing_ready}]}).
+
+-rabbit_boot_step({connection_tracking,
+                   [{description, "sets up internal storage for node-local connections"},
+                    {mfa,         {rabbit_connection_tracking, boot, []}},
+                    {requires,    routing_ready}]}).
 
 -rabbit_boot_step({networking,
                    [{mfa,         {rabbit_networking, boot, []}},
-                    {requires,    log_relay}]}).
+                    {requires,    routing_ready}]}).
 
 -rabbit_boot_step({notify_cluster,
                    [{description, "notify cluster nodes"},
@@ -200,6 +215,11 @@
                                    [rabbit_core_metrics_gc]}},
                     {enables,     networking}]}).
 
+-rabbit_boot_step({rabbit_looking_glass,
+                   [{description, "Looking Glass tracer and profiler"},
+                    {mfa,         {rabbit_looking_glass, boot, []}},
+                    {requires,    networking}]}).
+
 %%---------------------------------------------------------------------------
 
 -include("rabbit_framing.hrl").
@@ -211,9 +231,9 @@
 
 %%----------------------------------------------------------------------------
 
--type file_suffix() :: binary().
+-type restart_type() :: 'permanent' | 'transient' | 'temporary'.
 %% this really should be an abstract type
--type log_location() :: 'tty' | 'undefined' | file:filename().
+-type log_location() :: string().
 -type param() :: atom().
 -type app_name() :: atom().
 
@@ -231,10 +251,10 @@
 -spec is_running() -> boolean().
 -spec is_running(node()) -> boolean().
 -spec environment() -> [{param(), term()}].
--spec rotate_logs(file_suffix()) -> rabbit_types:ok_or_error(any()).
+-spec rotate_logs() -> rabbit_types:ok_or_error(any()).
 -spec force_event_refresh(reference()) -> 'ok'.
 
--spec log_location('sasl' | 'kernel') -> log_location().
+-spec log_locations() -> [log_location()].
 
 -spec start('normal',[]) ->
           {'error',
@@ -248,6 +268,8 @@
 -spec boot_delegate() -> 'ok'.
 -spec recover() -> 'ok'.
 -spec start_apps([app_name()]) -> 'ok'.
+-spec start_apps([app_name()],
+                 #{app_name() => restart_type()}) -> 'ok'.
 -spec stop_apps([app_name()]) -> 'ok'.
 
 %%----------------------------------------------------------------------------
@@ -267,7 +289,7 @@ start() ->
                      %% restarting the app.
                      ok = ensure_application_loaded(),
                      HipeResult = rabbit_hipe:maybe_hipe_compile(),
-                     ok = ensure_working_log_handlers(),
+                     ok = start_logger(),
                      rabbit_hipe:log_hipe_result(HipeResult),
                      rabbit_node_monitor:prepare_cluster_status_files(),
                      rabbit_mnesia:check_cluster_consistency(),
@@ -276,9 +298,10 @@ start() ->
 
 boot() ->
     start_it(fun() ->
+                     ensure_config(),
                      ok = ensure_application_loaded(),
                      HipeResult = rabbit_hipe:maybe_hipe_compile(),
-                     ok = ensure_working_log_handlers(),
+                     ok = start_logger(),
                      rabbit_hipe:log_hipe_result(HipeResult),
                      rabbit_node_monitor:prepare_cluster_status_files(),
                      ok = rabbit_upgrade:maybe_upgrade_mnesia(),
@@ -289,12 +312,32 @@ boot() ->
                      broker_start()
              end).
 
+ensure_config() ->
+    case rabbit_config:validate_config_files() of
+        ok -> ok;
+        {error, {ErrFmt, ErrArgs}} ->
+            log_boot_error_and_exit(check_config_file, ErrFmt, ErrArgs)
+    end,
+    case rabbit_config:prepare_and_use_config() of
+        {error, Reason} ->
+            {Format, Arg} = case Reason of
+                {generation_error, Error} -> {"~s", [Error]};
+                Other                     -> {"~p", [Other]}
+            end,
+            log_boot_error_and_exit(generate_config_file,
+                                    "~nConfig file generation failed "++Format,
+                                    Arg);
+        ok -> ok
+    end.
+
+
 broker_start() ->
     Plugins = rabbit_plugins:setup(),
     ToBeLoaded = Plugins ++ ?APPS,
     start_apps(ToBeLoaded),
     maybe_sd_notify(),
-    ok = log_broker_started(rabbit_plugins:active()).
+    ok = rabbit_lager:broker_is_started(),
+    ok = log_broker_started(rabbit_plugins:strictly_plugins(rabbit_plugins:active())).
 
 %% Try to send systemd ready notification if it makes sense in the
 %% current environment. standard_error is used intentionally in all
@@ -374,7 +417,8 @@ sd_notify_socat(Unit) ->
     end.
 
 sd_current_unit() ->
-    case catch re:run(os:cmd("systemctl status " ++ os:getpid()), "([-.@0-9a-zA-Z]+)", [unicode, {capture, all_but_first, list}]) of
+    CmdOut = os:cmd("ps -o unit= -p " ++ os:getpid()),
+    case catch re:run(CmdOut, "([-.@0-9a-zA-Z]+)", [unicode, {capture, all_but_first, list}]) of
         {'EXIT', _} ->
             error;
         {match, [Unit]} ->
@@ -397,7 +441,7 @@ sd_wait_activation(_, _, 0) ->
     io:format(standard_error, "Service still in 'activating' state, bailing out~n", []),
     false;
 sd_wait_activation(Port, Unit, AttemptsLeft) ->
-    case os:cmd("systemctl show --property=ActiveState " ++ Unit) of
+    case os:cmd("systemctl show --property=ActiveState -- '" ++ Unit ++ "'") of
         "ActiveState=activating\n" ->
             timer:sleep(1000),
             sd_wait_activation(Port, Unit, AttemptsLeft - 1);
@@ -417,10 +461,8 @@ start_it(StartFun) ->
                         false -> StartFun()
                     end
                 catch
-                    throw:{could_not_start, _App, _Reason} = Err ->
-                        boot_error(Err, not_available);
-                    _:Reason ->
-                        boot_error(Reason, erlang:get_stacktrace())
+                    Class:Reason ->
+                        boot_error(Class, Reason)
                 after
                     unlink(Marker),
                     Marker ! stop,
@@ -436,10 +478,12 @@ stop() ->
         undefined -> ok;
         _         ->
             rabbit_log:info("RabbitMQ hasn't finished starting yet. Waiting for startup to finish before stopping..."),
-            wait_for_boot_to_finish()
+            ok = wait_for_boot_to_finish(node())
     end,
     rabbit_log:info("RabbitMQ is asked to stop...~n", []),
     Apps = ?APPS ++ rabbit_plugins:active(),
+    %% this will also perform unregistration with the peer discovery backend
+    %% as needed
     stop_apps(app_utils:app_dependency_order(Apps, true)),
     rabbit_log:info("Successfully stopped RabbitMQ and its dependencies~n", []).
 
@@ -469,6 +513,9 @@ stop_and_halt() ->
     ok.
 
 start_apps(Apps) ->
+    start_apps(Apps, #{}).
+
+start_apps(Apps, RestartTypes) ->
     app_utils:load_applications(Apps),
 
     ConfigEntryDecoder = case application:get_env(rabbit, config_entry_decoder) of
@@ -481,7 +528,7 @@ start_apps(Apps) ->
         prompt ->
             IoDevice = get_input_iodevice(),
             io:setopts(IoDevice, [{echo, false}]),
-            PP = rabbit_misc:lists_droplast(io:get_line(IoDevice,
+            PP = lists:droplast(io:get_line(IoDevice,
                 "\nPlease enter the passphrase to unlock encrypted "
                 "configuration entries.\n\nPassphrase: ")),
             io:setopts(IoDevice, [{echo, true}]),
@@ -508,7 +555,8 @@ start_apps(Apps) ->
         true  -> ok                    %% will run during start of rabbit app
     end,
     ok = app_utils:start_applications(OrderedApps,
-                                      handle_app_error(could_not_start)).
+                                      handle_app_error(could_not_start),
+                                      RestartTypes).
 
 %% This function retrieves the correct IoDevice for requesting
 %% input. The problem with using the default IoDevice is that
@@ -582,9 +630,11 @@ decrypt_list([{Key, Value}|Tail], Algo, Acc) when Key =/= encrypted ->
 decrypt_list([Value|Tail], Algo, Acc) ->
     decrypt_list(Tail, Algo, [decrypt(Value, Algo)|Acc]).
 
+stop_apps([]) ->
+    ok;
 stop_apps(Apps) ->
     rabbit_log:info(
-        lists:flatten(["Stopping RabbitMQ applications and their dependencies in the following order: ~n",
+        lists:flatten(["Stopping RabbitMQ applications and their dependencies in the following order:~n",
                        ["    ~p~n" || _ <- Apps]]),
         lists:reverse(Apps)),
     ok = app_utils:stop_applications(
@@ -604,32 +654,52 @@ handle_app_error(Term) ->
     end.
 
 await_startup() ->
-    case is_booting() of
-        true -> wait_for_boot_to_finish();
+    await_startup(node()).
+
+await_startup(Node) ->
+    case is_booting(Node) of
+        true -> wait_for_boot_to_finish(Node);
         false ->
-            case is_running() of
+            case is_running(Node) of
                 true -> ok;
-                false -> wait_for_boot_to_start(),
-                         wait_for_boot_to_finish()
+                false -> wait_for_boot_to_start(Node),
+                         wait_for_boot_to_finish(Node)
             end
     end.
 
-is_booting() ->
-    whereis(rabbit_boot) /= undefined.
-
-wait_for_boot_to_start() ->
-    case whereis(rabbit_boot) of
-        undefined -> timer:sleep(100),
-                     wait_for_boot_to_start();
-        _         -> ok
+is_booting(Node) ->
+    case rpc:call(Node, erlang, whereis, [rabbit_boot]) of
+        {badrpc, _} = Err -> Err;
+        undefined         -> false;
+        P when is_pid(P)  -> true
     end.
 
-wait_for_boot_to_finish() ->
-    case whereis(rabbit_boot) of
-        undefined -> true = is_running(),
-                     ok;
-        _         -> timer:sleep(100),
-                     wait_for_boot_to_finish()
+wait_for_boot_to_start(Node) ->
+    case is_booting(Node) of
+        false ->
+            timer:sleep(100),
+            wait_for_boot_to_start(Node);
+        {badrpc, _} = Err ->
+            Err;
+        true  ->
+            ok
+    end.
+
+wait_for_boot_to_finish(Node) ->
+    case is_booting(Node) of
+        false ->
+            %% We don't want badrpc error to be interpreted as false,
+            %% so we don't call rabbit:is_running(Node)
+            case rpc:call(Node, rabbit, is_running, []) of
+                true              -> ok;
+                false             -> {error, rabbit_is_not_running};
+                {badrpc, _} = Err -> Err
+            end;
+        {badrpc, _} = Err ->
+            Err;
+        true  ->
+            timer:sleep(100),
+            wait_for_boot_to_finish(Node)
     end.
 
 status() ->
@@ -640,7 +710,8 @@ status() ->
           {erlang_version,       erlang:system_info(system_version)},
           {memory,               rabbit_vm:memory()},
           {alarms,               alarms()},
-          {listeners,            listeners()}],
+          {listeners,            listeners()},
+          {vm_memory_calculation_strategy, vm_memory_monitor:get_memory_calculation_strategy()}],
     S2 = rabbit_misc:filter_exit_map(
            fun ({Key, {M, F, A}}) -> {Key, erlang:apply(M, F, A)} end,
            [{vm_memory_high_watermark, {vm_memory_monitor,
@@ -690,6 +761,13 @@ is_running() -> is_running(node()).
 
 is_running(Node) -> rabbit_nodes:is_process_running(Node, rabbit).
 
+is_booted(Node) ->
+    case is_booting(Node) of
+        false ->
+            is_running(Node);
+        _ -> false
+    end.
+
 environment() ->
     %% The timeout value is twice that of gen_server:call/2.
     [{A, environment(A)} ||
@@ -700,27 +778,33 @@ environment(App) ->
     lists:keysort(1, [P || P = {K, _} <- application:get_all_env(App),
                            not lists:member(K, Ignore)]).
 
-rotate_logs_info("") ->
-    rabbit_log:info("Reopening logs", []);
-rotate_logs_info(Suffix) ->
-    rabbit_log:info("Rotating logs with suffix '~s'~n", [Suffix]).
-
-rotate_logs(BinarySuffix) ->
-    Suffix = binary_to_list(BinarySuffix),
-    rotate_logs_info(Suffix),
-    log_rotation_result(rotate_logs(log_location(kernel),
-                                    Suffix,
-                                    rabbit_error_logger_file_h),
-                        rotate_logs(log_location(sasl),
-                                    Suffix,
-                                    rabbit_sasl_report_file_h)).
+rotate_logs() ->
+    rabbit_lager:fold_sinks(
+      fun
+          (_, [], Acc) ->
+              Acc;
+          (SinkName, FileNames, Acc) ->
+              lager:log(SinkName, info, self(),
+                        "Log file rotation forced", []),
+              %% FIXME: We use an internal message, understood by
+              %% lager_file_backend. We should use a proper API, when
+              %% it's added to Lager.
+              %%
+              %% FIXME: This message is asynchronous, therefore this
+              %% entire call is asynchronous: at the end of this
+              %% function, we can't guaranty the rotation is completed.
+              [SinkName ! {rotate, FileName} || FileName <- FileNames],
+              lager:log(SinkName, info, self(),
+                        "Log file re-opened after forced rotation", []),
+              Acc
+      end, ok).
 
 %%--------------------------------------------------------------------
 
 start(normal, []) ->
     case erts_version_check() of
         ok ->
-            rabbit_log:info("Starting RabbitMQ ~s on Erlang ~s~n~s~n~s~n",
+            rabbit_log:info("~n Starting RabbitMQ ~s on Erlang ~s~n ~s~n ~s~n",
                             [rabbit_misc:version(), rabbit_misc:otp_release(),
                              ?COPYRIGHT_MESSAGE, ?INFORMATION_MESSAGE]),
             {ok, SupPid} = rabbit_sup:start_link(),
@@ -731,10 +815,24 @@ start(normal, []) ->
             warn_if_disc_io_options_dubious(),
             rabbit_boot_steps:run_boot_steps(),
             {ok, SupPid};
+        {error, {erlang_version_too_old,
+                 {found, OTPRel, ERTSVer},
+                 {required, ?OTP_MINIMUM, ?ERTS_MINIMUM}}} ->
+            Msg = "This RabbitMQ version cannot run on Erlang ~s (erts ~s): "
+                  "minimum required version is ~s (erts ~s)",
+            Args = [OTPRel, ERTSVer, ?OTP_MINIMUM, ?ERTS_MINIMUM],
+            rabbit_log:error(Msg, Args),
+            %% also print to stderr to make this more visible
+            io:format(standard_error, "Error: " ++ Msg ++ "~n", Args),
+            {error, {erlang_version_too_old, rabbit_misc:format("Erlang ~s or later is required, started on ~s", [?OTP_MINIMUM, OTPRel])}};
         Error ->
             Error
     end.
 
+prep_stop(State) ->
+  rabbit_peer_discovery:maybe_unregister(),
+  State.
+
 stop(_State) ->
     ok = rabbit_alarm:stop(),
     ok = case rabbit_mnesia:is_clustered() of
@@ -745,8 +843,7 @@ stop(_State) ->
 
 -spec boot_error(term(), not_available | [tuple()]) -> no_return().
 
-boot_error({could_not_start, rabbit, {{timeout_waiting_for_tables, _}, _}},
-           _Stacktrace) ->
+boot_error(_, {could_not_start, rabbit, {{timeout_waiting_for_tables, _}, _}}) ->
     AllNodes = rabbit_mnesia:cluster_nodes(all),
     Suffix = "~nBACKGROUND~n==========~n~n"
         "This cluster node was shut down while other nodes were still running.~n"
@@ -765,25 +862,25 @@ boot_error({could_not_start, rabbit, {{timeout_waiting_for_tables, _}, _}},
         end,
     log_boot_error_and_exit(
       timeout_waiting_for_tables,
-      Err ++ rabbit_nodes:diagnostics(Nodes) ++ "~n~n", []);
-boot_error(Reason, Stacktrace) ->
-    Fmt = "Error description:~n   ~p~n~n"
-        "Log files (may contain more information):~n   ~s~n   ~s~n~n",
-    Args = [Reason, log_location(kernel), log_location(sasl)],
-    boot_error(Reason, Fmt, Args, Stacktrace).
-
--spec boot_error(term(), string(), [any()], not_available | [tuple()]) ->
-          no_return().
-
-boot_error(Reason, Fmt, Args, not_available) ->
-    log_boot_error_and_exit(Reason, Fmt, Args);
-boot_error(Reason, Fmt, Args, Stacktrace) ->
-    log_boot_error_and_exit(Reason, Fmt ++ "Stack trace:~n   ~p~n~n",
-                            Args ++ [Stacktrace]).
+      "~n" ++ Err ++ rabbit_nodes:diagnostics(Nodes), []);
+boot_error(Class, {error, {cannot_log_to_file, _, _}} = Reason) ->
+    log_boot_error_and_exit(
+      Reason,
+      "~nError description:~s",
+      [lager:pr_stacktrace(erlang:get_stacktrace(), {Class, Reason})]);
+boot_error(Class, Reason) ->
+    LogLocations = log_locations(),
+    log_boot_error_and_exit(
+      Reason,
+      "~nError description:~s"
+      "~nLog file(s) (may contain more information):~n" ++
+      lists:flatten(["   ~s~n" || _ <- lists:seq(1, length(LogLocations))]),
+      [lager:pr_stacktrace(erlang:get_stacktrace(), {Class, Reason})] ++
+      LogLocations).
 
 log_boot_error_and_exit(Reason, Format, Args) ->
-    io:format("~n~nBOOT FAILED~n===========~n~n" ++ Format, Args),
-    rabbit_log:info(Format, Args),
+    rabbit_log:error(Format, Args),
+    io:format(standard_error, "~nBOOT FAILED~n===========~n" ++ Format ++ "~n", Args),
     timer:sleep(1000),
     exit(Reason).
 
@@ -796,10 +893,7 @@ boot_delegate() ->
 
 recover() ->
     rabbit_policy:recover(),
-    Qs = rabbit_amqqueue:recover(),
-    ok = rabbit_binding:recover(rabbit_exchange:recover(),
-                                [QName || #amqqueue{name = QName} <- Qs]),
-    rabbit_amqqueue:start(Qs).
+    rabbit_vhost:recover().
 
 maybe_insert_default_data() ->
     case rabbit_table:needs_default_data() of
@@ -822,91 +916,31 @@ insert_default_data() ->
     DefaultWritePermBin = rabbit_data_coercion:to_binary(DefaultWritePerm),
     DefaultReadPermBin = rabbit_data_coercion:to_binary(DefaultReadPerm),
 
-    ok = rabbit_vhost:add(DefaultVHostBin),
+    ok = rabbit_vhost:add(DefaultVHostBin, ?INTERNAL_USER),
     ok = rabbit_auth_backend_internal:add_user(
         DefaultUserBin,
-        DefaultPassBin
+        DefaultPassBin,
+        ?INTERNAL_USER
     ),
-    ok = rabbit_auth_backend_internal:set_tags(DefaultUserBin,DefaultTags),
+    ok = rabbit_auth_backend_internal:set_tags(DefaultUserBin, DefaultTags,
+                                               ?INTERNAL_USER),
     ok = rabbit_auth_backend_internal:set_permissions(DefaultUserBin,
                                                       DefaultVHostBin,
                                                       DefaultConfigurePermBin,
                                                       DefaultWritePermBin,
-                                                      DefaultReadPermBin),
+                                                      DefaultReadPermBin,
+                                                      ?INTERNAL_USER),
     ok.
 
 %%---------------------------------------------------------------------------
 %% logging
 
-ensure_working_log_handlers() ->
-    Handlers = gen_event:which_handlers(error_logger),
-    ok = ensure_working_log_handler(error_logger_tty_h,
-                                    rabbit_error_logger_file_h,
-                                    error_logger_tty_h,
-                                    log_location(kernel),
-                                    Handlers),
-
-    ok = ensure_working_log_handler(sasl_report_tty_h,
-                                    rabbit_sasl_report_file_h,
-                                    sasl_report_tty_h,
-                                    log_location(sasl),
-                                    Handlers),
+start_logger() ->
+    rabbit_lager:start_logger(),
     ok.
 
-ensure_working_log_handler(OldHandler, NewHandler, TTYHandler,
-                           LogLocation, Handlers) ->
-    case LogLocation of
-        undefined -> ok;
-        tty       -> case lists:member(TTYHandler, Handlers) of
-                         true  -> ok;
-                         false ->
-                             throw({error, {cannot_log_to_tty,
-                                            TTYHandler, not_installed}})
-                     end;
-        _         -> case lists:member(NewHandler, Handlers) of
-                         true  -> ok;
-                         false -> case rotate_logs(LogLocation, "",
-                                                   OldHandler, NewHandler) of
-                                      ok -> ok;
-                                      {error, Reason} ->
-                                          throw({error, {cannot_log_to_file,
-                                                         LogLocation, Reason}})
-                                  end
-                     end
-    end.
-
-log_location(Type) ->
-    case application:get_env(rabbit, case Type of
-                                         kernel -> error_logger;
-                                         sasl   -> sasl_error_logger
-                                     end) of
-        {ok, {file, File}} -> File;
-        {ok, false}        -> undefined;
-        {ok, tty}          -> tty;
-        {ok, silent}       -> undefined;
-        {ok, Bad}          -> throw({error, {cannot_log_to_file, Bad}});
-        _                  -> undefined
-    end.
-
-rotate_logs(File, Suffix, Handler) ->
-    rotate_logs(File, Suffix, Handler, Handler).
-
-rotate_logs(undefined, _Suffix, _OldHandler, _NewHandler) -> ok;
-rotate_logs(tty,       _Suffix, _OldHandler, _NewHandler) -> ok;
-rotate_logs(File,       Suffix,  OldHandler,  NewHandler) ->
-    gen_event:swap_handler(error_logger,
-                           {OldHandler, swap},
-                           {NewHandler, {File, Suffix}}).
-
-log_rotation_result({error, MainLogError}, {error, SaslLogError}) ->
-    {error, {{cannot_rotate_main_logs, MainLogError},
-             {cannot_rotate_sasl_logs, SaslLogError}}};
-log_rotation_result({error, MainLogError}, ok) ->
-    {error, {cannot_rotate_main_logs, MainLogError}};
-log_rotation_result(ok, {error, SaslLogError}) ->
-    {error, {cannot_rotate_sasl_logs, SaslLogError}};
-log_rotation_result(ok, ok) ->
-    ok.
+log_locations() ->
+    rabbit_lager:log_locations().
 
 force_event_refresh(Ref) ->
     rabbit_direct:force_event_refresh(Ref),
@@ -918,19 +952,17 @@ force_event_refresh(Ref) ->
 %% misc
 
 log_broker_started(Plugins) ->
-    rabbit_log:with_local_io(
-      fun() ->
-              PluginList = iolist_to_binary([rabbit_misc:format(" * ~s~n", [P])
-                                             || P <- Plugins]),
-              rabbit_log:info(
-                "Server startup complete; ~b plugins started.~n~s",
-                [length(Plugins), PluginList]),
-              io:format(" completed with ~p plugins.~n", [length(Plugins)])
-      end).
+    PluginList = iolist_to_binary([rabbit_misc:format(" * ~s~n", [P])
+                                   || P <- Plugins]),
+    Message = string:strip(rabbit_misc:format(
+        "Server startup complete; ~b plugins started.~n~s",
+        [length(Plugins), PluginList]), right, $\n),
+    rabbit_log:info(Message),
+    io:format(" completed with ~p plugins.~n", [length(Plugins)]).
 
 erts_version_check() ->
     ERTSVer = erlang:system_info(version),
-    OTPRel = erlang:system_info(otp_release),
+    OTPRel = rabbit_misc:otp_release(),
     case rabbit_misc:version_compare(?ERTS_MINIMUM, ERTSVer, lte) of
         true when ?ERTS_MINIMUM =/= ERTSVer ->
             ok;
@@ -951,31 +983,45 @@ erts_version_check() ->
 print_banner() ->
     {ok, Product} = application:get_key(description),
     {ok, Version} = application:get_key(vsn),
-    io:format("~n              ~s ~s. ~s"
-              "~n  ##  ##      ~s"
-              "~n  ##  ##"
-              "~n  ##########  Logs: ~s"
-              "~n  ######  ##        ~s"
-              "~n  ##########"
-              "~n              Starting broker..."
+    {LogFmt, LogLocations} = case log_locations() of
+        [_ | Tail] = LL ->
+            LF = lists:flatten(["~n                    ~s"
+                                || _ <- lists:seq(1, length(Tail))]),
+            {LF, LL};
+        [] ->
+            {"", ["(none)"]}
+    end,
+    io:format("~n  ##  ##"
+              "~n  ##  ##      ~s ~s. ~s"
+              "~n  ##########  ~s"
+              "~n  ######  ##"
+              "~n  ##########  Logs: ~s" ++
+              LogFmt ++
+              "~n~n              Starting broker..."
               "~n",
-              [Product, Version, ?COPYRIGHT_MESSAGE, ?INFORMATION_MESSAGE,
-               log_location(kernel), log_location(sasl)]).
+              [Product, Version, ?COPYRIGHT_MESSAGE, ?INFORMATION_MESSAGE] ++
+              LogLocations).
 
 log_banner() ->
+    {FirstLog, OtherLogs} = case log_locations() of
+        [Head | Tail] ->
+            {Head, [{"", F} || F <- Tail]};
+        [] ->
+            {"(none)", []}
+    end,
     Settings = [{"node",           node()},
                 {"home dir",       home_dir()},
                 {"config file(s)", config_files()},
                 {"cookie hash",    rabbit_nodes:cookie_hash()},
-                {"log",            log_location(kernel)},
-                {"sasl log",       log_location(sasl)},
-                {"database dir",   rabbit_mnesia:dir()}],
+                {"log(s)",         FirstLog}] ++
+               OtherLogs ++
+               [{"database dir",   rabbit_mnesia:dir()}],
     DescrLen = 1 + lists:max([length(K) || {K, _V} <- Settings]),
     Format = fun (K, V) ->
                      rabbit_misc:format(
-                       "~-" ++ integer_to_list(DescrLen) ++ "s: ~s~n", [K, V])
+                       " ~-" ++ integer_to_list(DescrLen) ++ "s: ~s~n", [K, V])
              end,
-    Banner = iolist_to_binary(
+    Banner = string:strip(lists:flatten(
                [case S of
                     {"config file(s)" = K, []} ->
                         Format(K, "(none)");
@@ -983,8 +1029,8 @@ log_banner() ->
                         [Format(K, V0) | [Format("", V) || V <- Vs]];
                     {K, V} ->
                         Format(K, V)
-                end || S <- Settings]),
-    rabbit_log:info("~s", [Banner]).
+                end || S <- Settings]), right, $\n),
+    rabbit_log:info("~n~s", [Banner]).
 
 warn_if_kernel_config_dubious() ->
     case os:type() of
@@ -1102,32 +1148,7 @@ home_dir() ->
     end.
 
 config_files() ->
-    Abs = fun (F) ->
-                  filename:absname(filename:rootname(F, ".config") ++ ".config")
-          end,
-    case init:get_argument(config) of
-        {ok, Files} -> [Abs(File) || [File] <- Files];
-        error       -> case config_setting() of
-                           none -> [];
-                           File -> [Abs(File) ++ " (not found)"]
-                       end
-    end.
-
-%% This is a pain. We want to know where the config file is. But we
-%% can't specify it on the command line if it is missing or the VM
-%% will fail to start, so we need to find it by some mechanism other
-%% than init:get_arguments/0. We can look at the environment variable
-%% which is responsible for setting it... but that doesn't work for a
-%% Windows service since the variable can change and the service not
-%% be reinstalled, so in that case we add a magic application env.
-config_setting() ->
-    case application:get_env(rabbit, windows_service_config) of
-        {ok, File1} -> File1;
-        undefined   -> case os:getenv("RABBITMQ_CONFIG_FILE") of
-                           false -> none;
-                           File2 -> File2
-                       end
-    end.
+    rabbit_config:config_files().
 
 %% We don't want this in fhc since it references rabbit stuff. And we can't put
 %% this in the bootstep directly.
@@ -1153,9 +1174,8 @@ ensure_working_fhc() ->
             {ok, true}  -> "ON";
             {ok, false} -> "OFF"
         end,
-        rabbit_log:info(
-          "FHC read buffering:  ~s~n"
-          "FHC write buffering: ~s~n", [ReadBuf, WriteBuf]),
+        rabbit_log:info("FHC read buffering:  ~s~n", [ReadBuf]),
+        rabbit_log:info("FHC write buffering: ~s~n", [WriteBuf]),
         Filename = filename:join(code:lib_dir(kernel, ebin), "kernel.app"),
         {ok, Fd} = file_handle_cache:open(Filename, [raw, binary, read], []),
         {ok, _} = file_handle_cache:read(Fd, 1),
