@@ -23,6 +23,7 @@
 -export([validate/1, validate_binding/2,
          create/2, delete/3, policy_changed/2, add_binding/3,
          remove_bindings/3, assert_args_equivalence/2]).
+-export([info/1, info/2]).
 
 -rabbit_boot_step({?MODULE,
                    [{description, "exchange type direct"},
@@ -31,6 +32,9 @@
                     {requires,    rabbit_registry},
                     {enables,     kernel_ready}]}).
 
+info(_X) -> [].
+info(_X, _) -> [].
+
 description() ->
     [{description, <<"AMQP direct exchange, as per the AMQP specification">>}].
 
