@@ -24,6 +24,7 @@
 -export([validate/1, validate_binding/2,
          create/2, delete/3, policy_changed/2, add_binding/3,
          remove_bindings/3, assert_args_equivalence/2]).
+-export([info/1, info/2]).
 
 -rabbit_boot_step({?MODULE,
                    [{description, "exchange type headers"},
@@ -36,6 +37,9 @@
         (rabbit_framing:amqp_table(), rabbit_framing:amqp_table()) ->
             boolean().
 
+info(_X) -> [].
+info(_X, _) -> [].
+
 description() ->
     [{description, <<"AMQP headers exchange, as per the AMQP specification">>}].
 
