@@ -24,6 +24,7 @@
 -export([validate/1, validate_binding/2,
          create/2, delete/3, policy_changed/2, add_binding/3,
          remove_bindings/3, assert_args_equivalence/2]).
+-export([info/1, info/2]).
 
 -rabbit_boot_step({?MODULE,
                    [{description, "exchange type topic"},
@@ -34,6 +35,9 @@
 
 %%----------------------------------------------------------------------------
 
+info(_X) -> [].
+info(_X, _) -> [].
+
 description() ->
     [{description, <<"AMQP topic exchange, as per the AMQP specification">>}].
 
