@@ -16,13 +16,19 @@
 
 -module(rabbit_parameter_validation).
 
--export([number/2, binary/2, boolean/2, list/2, regex/2, proplist/3, enum/1]).
+-export([number/2, integer/2, binary/2, boolean/2, list/2, regex/2, proplist/3, enum/1]).
 
 number(_Name, Term) when is_number(Term) ->
     ok;
 
 number(Name, Term) ->
-    {error, "~s should be number, actually was ~p", [Name, Term]}.
+    {error, "~s should be a number, actually was ~p", [Name, Term]}.
+
+integer(_Name, Term) when is_integer(Term) ->
+    ok;
+
+integer(Name, Term) ->
+    {error, "~s should be a number, actually was ~p", [Name, Term]}.
 
 binary(_Name, Term) when is_binary(Term) ->
     ok;
@@ -71,6 +77,10 @@ proplist(Name, Constraints, Term) when is_list(Term) ->
                | Results]
     end;
 
+proplist(Name, Constraints, Term0) when is_map(Term0) ->
+    Term = maps:to_list(Term0),
+    proplist(Name, Constraints, Term);
+
 proplist(Name, _Constraints, Term) ->
     {error, "~s not a list ~p", [Name, Term]}.
 
