@@ -21,6 +21,9 @@
          check_schema_integrity/1, clear_ram_only_tables/0, retry_timeout/0,
          wait_for_replicated/0]).
 
+%% for testing purposes
+-export([definitions/0]).
+
 -include("rabbit.hrl").
 
 %%----------------------------------------------------------------------------
@@ -53,8 +56,20 @@ create() ->
                                                  Tab, TabDef1, Reason}})
                           end
                   end, definitions()),
+    ensure_secondary_indexes(),
     ok.
 
+%% Sets up secondary indexes in a blank node database.
+ensure_secondary_indexes() ->
+  ensure_secondary_index(rabbit_queue, vhost),
+  ok.
+
+ensure_secondary_index(Table, Field) ->
+  case mnesia:add_table_index(Table, Field) of
+    {atomic, ok}                          -> ok;
+    {aborted, {already_exists, Table, _}} -> ok
+  end.
+
 %% The sequence in which we delete the schema and then the other
 %% tables is important: if we delete the schema first when moving to
 %% RAM mnesia will loudly complain since it doesn't make much sense to
@@ -260,6 +275,13 @@ definitions() ->
        {match, #user_permission{user_vhost = #user_vhost{_='_'},
                                 permission = #permission{_='_'},
                                 _='_'}}]},
+     {rabbit_topic_permission,
+      [{record_name, topic_permission},
+       {attributes, record_info(fields, topic_permission)},
+       {disc_copies, [node()]},
+       {match, #topic_permission{topic_permission_key = #topic_permission_key{_='_'},
+                                 permission = #permission{_='_'},
+                                 _='_'}}]},
      {rabbit_vhost,
       [{record_name, vhost},
        {attributes, record_info(fields, vhost)},
