@@ -49,10 +49,11 @@ start_link(IPAddress, Port, Transport, SocketOpts, ProtoSup, ProtoOpts, OnStartu
 init({IPAddress, Port, Transport, SocketOpts, ProtoSup, ProtoOpts, OnStartup, OnShutdown,
       ConcurrentAcceptorCount, Label}) ->
     {ok, AckTimeout} = application:get_env(rabbit, ssl_handshake_timeout),
+    MaxConnections = rabbit_misc:get_env(rabbit, connection_max, infinity),
     {ok, {{one_for_all, 10, 10}, [
         ranch:child_spec({acceptor, IPAddress, Port}, ConcurrentAcceptorCount,
             Transport, [{port, Port}, {ip, IPAddress},
-                {max_connections, infinity},
+                {max_connections, MaxConnections},
                 {ack_timeout, AckTimeout},
                 {connection_type, supervisor}|SocketOpts],
             ProtoSup, ProtoOpts),
