@@ -31,8 +31,8 @@
 %% here rather than just a message-reflector. That's also why we pick
 %% the delegate process to use based on a hash of the source pid.
 %%
-%% When a function is invoked using delegate:invoke/2, delegate:call/2
-%% or delegate:cast/2 on a group of pids, the pids are first split
+%% When a function is invoked using delegate:invoke/2,
+%% or delegate:invoke_no_result/2 on a group of pids, the pids are first split
 %% into local and remote ones. Remote processes are then grouped by
 %% node. The function is then invoked locally and on every node (using
 %% gen_server2:multi/4) as many times as there are processes on that
@@ -47,9 +47,12 @@
 
 -behaviour(gen_server2).
 
--export([start_link/1, start_link/2,invoke_no_result/2, invoke_no_result/3,
-         invoke/2, invoke/3, monitor/2, monitor/3, demonitor/1,
-         call/2, cast/2, call/3, cast/3]).
+%% Transitional step until we can require Erlang/OTP 21 and
+%% use the now recommended try/catch syntax for obtaining the stack trace.
+-compile(nowarn_deprecated_function).
+
+-export([start_link/1, start_link/2, invoke_no_result/2,
+         invoke/2, invoke/3, monitor/2, monitor/3, demonitor/1]).
 
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2,
          terminate/2, code_change/3]).
@@ -68,15 +71,10 @@
 -spec invoke
         ( pid(),  fun_or_mfa(A)) -> A;
         ([pid()], fun_or_mfa(A)) -> {[{pid(), A}], [{pid(), term()}]}.
--spec invoke_no_result(pid() | [pid()], fun_or_mfa(any())) -> 'ok'.
+-spec invoke_no_result(pid(), fun_or_mfa(any())) -> 'ok'.
 -spec monitor('process', pid()) -> monitor_ref().
 -spec demonitor(monitor_ref()) -> 'true'.
 
--spec call
-        ( pid(),  any()) -> any();
-        ([pid()], any()) -> {[{pid(), any()}], [{pid(), term()}]}.
--spec cast(pid() | [pid()], any()) -> 'ok'.
-
 %%----------------------------------------------------------------------------
 
 -define(HIBERNATE_AFTER_MIN, 1000).
@@ -118,7 +116,7 @@ invoke(Pids, Name, FunOrMFA) when is_list(Pids) ->
     %% infinity, and thus there is no process spawned in order to do
     %% the sending. Thus calls can't overtake preceding calls/casts.
     {Replies, BadNodes} =
-        case orddict:fetch_keys(Grouped) of
+        case maps:keys(Grouped) of
             []          -> {[], []};
             RemoteNodes -> gen_server2:multi_call(
                              RemoteNodes, delegate(self(), Name, RemoteNodes),
@@ -126,7 +124,7 @@ invoke(Pids, Name, FunOrMFA) when is_list(Pids) ->
         end,
     BadPids = [{Pid, {exit, {nodedown, BadNode}, []}} ||
                   BadNode <- BadNodes,
-                  Pid     <- orddict:fetch(BadNode, Grouped)],
+                  Pid     <- maps:get(BadNode, Grouped)],
     ResultsNoNode = lists:append([safe_invoke(LocalPids, FunOrMFA) |
                                   [Results || {_Node, Results} <- Replies]]),
     lists:foldl(
@@ -134,31 +132,6 @@ invoke(Pids, Name, FunOrMFA) when is_list(Pids) ->
           ({error, Pid, Error},  {Good, Bad}) -> {Good, [{Pid, Error} | Bad]}
       end, {[], BadPids}, ResultsNoNode).
 
-invoke_no_result(Pid, FunOrMFA) ->
-    invoke_no_result(Pid, ?DEFAULT_NAME, FunOrMFA).
-
-invoke_no_result(Pid, _Name, FunOrMFA) when is_pid(Pid) andalso node(Pid) =:= node() ->
-    _ = safe_invoke(Pid, FunOrMFA), %% we don't care about any error
-    ok;
-invoke_no_result(Pid, Name, FunOrMFA) when is_pid(Pid) ->
-    invoke_no_result([Pid], Name, FunOrMFA);
-
-invoke_no_result([], _Name, _FunOrMFA) -> %% optimisation
-    ok;
-invoke_no_result([Pid], _Name, FunOrMFA) when node(Pid) =:= node() -> %% optimisation
-    _ = safe_invoke(Pid, FunOrMFA), %% must not die
-    ok;
-invoke_no_result(Pids, Name, FunOrMFA) when is_list(Pids) ->
-    {LocalPids, Grouped} = group_pids_by_node(Pids),
-    case orddict:fetch_keys(Grouped) of
-        []          -> ok;
-        RemoteNodes -> gen_server2:abcast(
-                         RemoteNodes, delegate(self(), Name, RemoteNodes),
-                         {invoke, FunOrMFA, Grouped})
-    end,
-    _ = safe_invoke(LocalPids, FunOrMFA), %% must not die
-    ok.
-
 monitor(process, Pid) ->
     ?MODULE:monitor(process, Pid, ?DEFAULT_NAME).
 
@@ -174,17 +147,45 @@ demonitor(Ref) when is_reference(Ref) ->
 demonitor({Name, Pid}) ->
     gen_server2:cast(Name, {demonitor, self(), Pid}).
 
-call(PidOrPids, Name, Msg) ->
-    invoke(PidOrPids, Name, {gen_server2, call, [Msg, infinity]}).
-
-call(PidOrPids, Msg) ->
-    call(PidOrPids, ?DEFAULT_NAME, Msg).
-
-cast(PidOrPids, Msg) ->
-    cast(PidOrPids, ?DEFAULT_NAME, Msg).
-
-cast(PidOrPids, Name, Msg) ->
-    invoke_no_result(PidOrPids, Name, {gen_server2, cast, [Msg]}).
+invoke_no_result(Pid, FunOrMFA) when is_pid(Pid) andalso node(Pid) =:= node() ->
+    %% Optimization, avoids calling invoke_no_result/3.
+    %%
+    %% This may seem like a cosmetic change at first but it actually massively reduces the memory usage in mirrored
+    %% queues when ack/nack are sent to the node that hosts a mirror.
+    %% This way binary references are not kept around unnecessarily.
+    %%
+    %% See https://github.com/rabbitmq/rabbitmq-common/issues/208#issuecomment-311308583 for a before/after
+    %% comparison.
+    _ = safe_invoke(Pid, FunOrMFA), %% we don't care about any error
+    ok;
+invoke_no_result(Pid, FunOrMFA) when is_pid(Pid) ->
+    %% Optimization, avoids calling invoke_no_result/3
+    RemoteNode  = node(Pid),
+    gen_server2:abcast([RemoteNode], delegate(self(), ?DEFAULT_NAME, [RemoteNode]),
+                       {invoke, FunOrMFA,
+                        maps:from_list([{RemoteNode, [Pid]}])}),
+    ok;
+invoke_no_result([], _FunOrMFA) -> %% optimisation
+    ok;
+invoke_no_result([Pid], FunOrMFA) when node(Pid) =:= node() -> %% optimisation
+    _ = safe_invoke(Pid, FunOrMFA), %% must not die
+    ok;
+invoke_no_result([Pid], FunOrMFA) ->
+    RemoteNode  = node(Pid),
+    gen_server2:abcast([RemoteNode], delegate(self(), ?DEFAULT_NAME, [RemoteNode]),
+                       {invoke, FunOrMFA,
+                        maps:from_list([{RemoteNode, [Pid]}])}),
+    ok;
+invoke_no_result(Pids, FunOrMFA) when is_list(Pids) ->
+    {LocalPids, Grouped} = group_pids_by_node(Pids),
+    case maps:keys(Grouped) of
+        []          -> ok;
+        RemoteNodes -> gen_server2:abcast(
+                         RemoteNodes, delegate(self(), ?DEFAULT_NAME, RemoteNodes),
+                         {invoke, FunOrMFA, Grouped})
+    end,
+    _ = safe_invoke(LocalPids, FunOrMFA), %% must not die
+    ok.
 
 %%----------------------------------------------------------------------------
 
@@ -195,9 +196,9 @@ group_pids_by_node(Pids) ->
               {[Pid | Local], Remote};
           (Pid, {Local, Remote}) ->
               {Local,
-               orddict:update(
+               maps:update_with(
                  node(Pid), fun (List) -> [Pid | List] end, [Pid], Remote)}
-      end, {[], orddict:new()}, Pids).
+      end, {[], maps:new()}, Pids).
 
 delegate_name(Name, Hash) ->
     list_to_atom(Name ++ integer_to_list(Hash)).
@@ -231,7 +232,7 @@ init([Name]) ->
      {backoff, ?HIBERNATE_AFTER_MIN, ?HIBERNATE_AFTER_MIN, ?DESIRED_HIBERNATE}}.
 
 handle_call({invoke, FunOrMFA, Grouped}, _From, State = #state{node = Node}) ->
-    {reply, safe_invoke(orddict:fetch(Node, Grouped), FunOrMFA), State,
+    {reply, safe_invoke(maps:get(Node, Grouped), FunOrMFA), State,
      hibernate}.
 
 handle_cast({monitor, MonitoringPid, Pid},
@@ -263,7 +264,7 @@ handle_cast({demonitor, MonitoringPid, Pid},
     {noreply, State#state{monitors = Monitors1}, hibernate};
 
 handle_cast({invoke, FunOrMFA, Grouped}, State = #state{node = Node}) ->
-    _ = safe_invoke(orddict:fetch(Node, Grouped), FunOrMFA),
+    _ = safe_invoke(maps:get(Node, Grouped), FunOrMFA),
     {noreply, State, hibernate}.
 
 handle_info({'DOWN', Ref, process, Pid, Info},
