@@ -538,14 +538,14 @@ clear(Ref) ->
       end).
 
 set_maximum_since_use(MaximumAge) ->
-    Now = time_compat:monotonic_time(),
+    Now = erlang:monotonic_time(),
     case lists:foldl(
            fun ({{Ref, fhc_handle},
                  Handle = #handle { hdl = Hdl, last_used_at = Then }}, Rep) ->
                    case Hdl =/= closed andalso
-                        time_compat:convert_time_unit(Now - Then,
-                                                      native,
-                                                      micro_seconds)
+                        erlang:convert_time_unit(Now - Then,
+                                                 native,
+                                                 micro_seconds)
                           >= MaximumAge of
                        true  -> soft_close(Ref, Handle) orelse Rep;
                        false -> Rep
@@ -692,7 +692,7 @@ get_or_reopen(RefNewOrReopens) ->
             {ok, [Handle || {_Ref, Handle} <- OpenHdls]};
         {OpenHdls, ClosedHdls} ->
             Oldest = oldest(get_age_tree(),
-                            fun () -> time_compat:monotonic_time() end),
+                            fun () -> erlang:monotonic_time() end),
             case gen_server2:call(?SERVER, {open, self(), length(ClosedHdls),
                                             Oldest}, infinity) of
                 ok ->
@@ -728,7 +728,7 @@ reopen([{Ref, NewOrReopen, Handle = #handle { hdl          = closed,
            end,
     case prim_file:open(Path, Mode) of
         {ok, Hdl} ->
-            Now = time_compat:monotonic_time(),
+            Now = erlang:monotonic_time(),
             {{ok, _Offset}, Handle1} =
                 maybe_seek(Offset, reset_read_buffer(
                                      Handle#handle{hdl              = Hdl,
@@ -764,7 +764,7 @@ sort_handles([{Ref, _} | RefHdls], RefHdlsA, [{Ref, Handle} | RefHdlsB], Acc) ->
     sort_handles(RefHdls, RefHdlsA, RefHdlsB, [Handle | Acc]).
 
 put_handle(Ref, Handle = #handle { last_used_at = Then }) ->
-    Now = time_compat:monotonic_time(),
+    Now = erlang:monotonic_time(),
     age_tree_update(Then, Now, Ref),
     put({Ref, fhc_handle}, Handle #handle { last_used_at = Now }).
 
@@ -1405,14 +1405,14 @@ reduce(State = #fhc_state { open_pending          = OpenPending,
                             elders                = Elders,
                             clients               = Clients,
                             timer_ref             = TRef }) ->
-    Now = time_compat:monotonic_time(),
+    Now = erlang:monotonic_time(),
     {CStates, Sum, ClientCount} =
         ets:foldl(fun ({Pid, Eldest}, {CStatesAcc, SumAcc, CountAcc} = Accs) ->
                           [#cstate { pending_closes = PendingCloses,
                                      opened         = Opened,
                                      blocked        = Blocked } = CState] =
                               ets:lookup(Clients, Pid),
-                          TimeDiff = time_compat:convert_time_unit(
+                          TimeDiff = erlang:convert_time_unit(
                             Now - Eldest, native, micro_seconds),
                           case Blocked orelse PendingCloses =:= Opened of
                               true  -> Accs;
@@ -1451,7 +1451,7 @@ notify_age(CStates, AverageAge) ->
 notify_age0(Clients, CStates, Required) ->
     case [CState || CState <- CStates, CState#cstate.callback =/= undefined] of
         []            -> ok;
-        Notifications -> S = rand_compat:uniform(length(Notifications)),
+        Notifications -> S = rand:uniform(length(Notifications)),
                          {L1, L2} = lists:split(S, Notifications),
                          notify(Clients, Required, L2 ++ L1)
     end.
@@ -1484,7 +1484,12 @@ track_client(Pid, Clients) ->
 %% To increase the number of file descriptors: on Windows set ERL_MAX_PORTS
 %% environment variable, on Linux set `ulimit -n`.
 ulimit() ->
-    case proplists:get_value(max_fds, erlang:system_info(check_io)) of
+    IOStats = case erlang:system_info(check_io) of
+        [Val | _] when is_list(Val) -> Val;
+        Val when is_list(Val)       -> Val;
+        _Other                      -> []
+    end,
+    case proplists:get_value(max_fds, IOStats) of
         MaxFds when is_integer(MaxFds) andalso MaxFds > 1 ->
             case os:type() of
                 {win32, _OsName} ->
