@@ -16,6 +16,10 @@
 
 -module(rabbit_auth_mechanism).
 
+-behaviour(rabbit_registry_class).
+
+-export([added_to_rabbit_registry/2, removed_from_rabbit_registry/1]).
+
 %% A description.
 -callback description() -> [proplists:property()].
 
@@ -41,3 +45,6 @@
     {'challenge', binary(), any()} |
     {'protocol_error', string(), [any()]} |
     {'refused', rabbit_types:username() | none, string(), [any()]}.
+
+added_to_rabbit_registry(_Type, _ModuleName) -> ok.
+removed_from_rabbit_registry(_Type) -> ok.
