@@ -30,7 +30,7 @@
 %%     Something went wrong. Log and die.
 %% {refused, Msg, Args}
 %%     Client failed authentication. Log and die.
--callback user_login_authentication(rabbit_types:username(), [term()]) ->
+-callback user_login_authentication(rabbit_types:username(), [term()] | map()) ->
     {'ok', rabbit_types:auth_user()} |
     {'refused', string(), [any()]} |
     {'error', any()}.
