@@ -19,6 +19,7 @@
 -include("rabbit_core_metrics.hrl").
 
 -export([init/0]).
+-export([terminate/0]).
 
 -export([connection_created/2,
          connection_closed/1,
@@ -39,7 +40,8 @@
 
 -export([queue_stats/2,
          queue_stats/5,
-         queue_deleted/1]).
+         queue_deleted/1,
+         queues_deleted/1]).
 
 -export([node_stats/2]).
 
@@ -51,31 +53,6 @@
 
 -export([delete/2]).
 
-%% Those functions are exported for internal use only, not for public
-%% consumption.
--export([
-         ets_update_counter/4,
-         ets_update_counter_pre_18/4,
-         ets_update_counter_post_18/4
-        ]).
-
--export([
-         ets_update_element/3,
-         ets_update_element_pre_18/3,
-         ets_update_element_post_18/3
-        ]).
-
--erlang_version_support([
-                         {18, [
-                               {ets_update_counter, 4,
-                                ets_update_counter_pre_18,
-                                ets_update_counter_post_18},
-                               {ets_update_element, 3,
-                                ets_update_element_pre_18,
-                                ets_update_element_post_18}
-                              ]}
-                        ]).
-
 %%----------------------------------------------------------------------------
 %% Types
 %%----------------------------------------------------------------------------
@@ -124,8 +101,14 @@
 %% API
 %%----------------------------------------------------------------------------
 init() ->
-    [ets:new(Table, [Type, public, named_table, {write_concurrency, true}])
-     || {Table, Type} <- ?CORE_TABLES ++ ?CORE_EXTRA_TABLES],
+    _ = [ets:new(Table, [Type, public, named_table, {write_concurrency, true},
+                         {read_concurrency, true}])
+         || {Table, Type} <- ?CORE_TABLES ++ ?CORE_EXTRA_TABLES],
+    ok.
+
+terminate() ->
+    [ets:delete(Table)
+     || {Table, _Type} <- ?CORE_TABLES ++ ?CORE_EXTRA_TABLES],
     ok.
 
 connection_created(Pid, Infos) ->
@@ -136,7 +119,7 @@ connection_closed(Pid) ->
     ets:delete(connection_created, Pid),
     ets:delete(connection_metrics, Pid),
     %% Delete marker
-    ets_update_element(connection_coarse_metrics, Pid, {5, 1}),
+    ets:update_element(connection_coarse_metrics, Pid, {5, 1}),
     ok.
 
 connection_stats(Pid, Infos) ->
@@ -168,117 +151,62 @@ channel_stats(reductions, Id, Value) ->
 
 channel_stats(exchange_stats, publish, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_exchange_metrics, Id, {2, Value}, {Id, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_exchange_metrics, Id, {2, Value}, {Id, 0, 0, 0, 0}),
     ok;
 channel_stats(exchange_stats, confirm, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_exchange_metrics, Id, {3, Value}, {Id, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_exchange_metrics, Id, {3, Value}, {Id, 0, 0, 0, 0}),
     ok;
 channel_stats(exchange_stats, return_unroutable, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_exchange_metrics, Id, {4, Value}, {Id, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_exchange_metrics, Id, {4, Value}, {Id, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_exchange_stats, publish, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_queue_exchange_metrics, Id, Value, {Id, 0, 0}),
+    _ = ets:update_counter(channel_queue_exchange_metrics, Id, Value, {Id, 0, 0}),
     ok;
 channel_stats(queue_stats, get, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_queue_metrics, Id, {2, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {2, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, get_no_ack, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_queue_metrics, Id, {3, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {3, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, deliver, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_queue_metrics, Id, {4, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {4, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, deliver_no_ack, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_queue_metrics, Id, {5, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {5, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, redeliver, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_queue_metrics, Id, {6, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {6, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
     ok;
 channel_stats(queue_stats, ack, Id, Value) ->
     %% Includes delete marker
-    ets_update_counter(channel_queue_metrics, Id, {7, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
+    _ = ets:update_counter(channel_queue_metrics, Id, {7, Value}, {Id, 0, 0, 0, 0, 0, 0, 0}),
     ok.
 
 delete(Table, Key) ->
-    ets:delete(Table, Key).
-
-%% ets:update_counter(Tab, Key, Incr, Default) appeared in Erlang 18.x.
-%% We need a wrapper for Erlang R16B03 and Erlang 17.x.
-
-ets_update_counter(Tab, Key, Incr, Default) ->
-    code_version:update(?MODULE),
-    ?MODULE:ets_update_counter(Tab, Key, Incr, Default).
-
-ets_update_counter_pre_18(Tab, Key, Incr, Default) ->
-    %% The wrapper tries to update the counter first. If it's missing
-    %% (and a `badarg` is raised), it inserts the default value and
-    %% tries to update the counter one more time.
-    try
-        ets:update_counter(Tab, Key, Incr)
-    catch
-        _:badarg ->
-            try
-                %% There is no atomicity here, so between the
-                %% call to `ets:insert_new/2` and the call to
-                %% `ets:update_counter/3`, the the counters have
-                %% a temporary value (which is not possible with
-                %% `ets:update_counter/4). Furthermore, there is a
-                %% chance for the counter to be removed between those
-                %% two calls as well.
-                ets:insert_new(Tab, Default),
-                ets:update_counter(Tab, Key, Incr)
-            catch
-                _:badarg ->
-                    %% We can't tell with just `badarg` what the real
-                    %% cause is. We have no way to decide if we should
-                    %% try to insert/update the counter again, so let's
-                    %% do nothing.
-                    0
-            end
-    end.
-
-ets_update_counter_post_18(Tab, Key, Incr, Default) ->
-    ets:update_counter(Tab, Key, Incr, Default).
-
-%% ets:update_element(Tab, Key, ElementSpec) appeared in Erlang 18.x.
-%% We need a wrapper for Erlang R16B03 and Erlang 17.x.
-
-ets_update_element(Tab, Key, ElementSpec) ->
-    code_version:update(?MODULE),
-    ?MODULE:ets_update_element(Tab, Key, ElementSpec).
-
-ets_update_element_pre_18(Tab, Key, {Pos, Value}) ->
-    case ets:lookup(Tab, Key) of
-        [] ->
-            ok;
-        [Tuple] ->
-            ets:insert(Tab, setelement(Pos, Tuple, Value))
-    end.
-
-ets_update_element_post_18(Tab, Key, ElementSpec) ->
-    ets:update_element(Tab, Key, ElementSpec).
+    ets:delete(Table, Key),
+    ok.
 
 channel_queue_down(Id) ->
     %% Delete marker
-    ets_update_element(channel_queue_metrics, Id, {8, 1}),
+    ets:update_element(channel_queue_metrics, Id, {8, 1}),
     ok.
 
 channel_queue_exchange_down(Id) ->
     %% Delete marker
-    ets_update_element(channel_queue_exchange_metrics, Id, {3, 1}),
+    ets:update_element(channel_queue_exchange_metrics, Id, {3, 1}),
     ok.
 
 channel_exchange_down(Id) ->
     %% Delete marker
-    ets_update_element(channel_exchange_metrics, Id, {5, 1}),
+    ets:update_element(channel_exchange_metrics, Id, {5, 1}),
     ok.
 
 consumer_created(ChPid, ConsumerTag, ExclusiveConsume, AckRequired, QName,
@@ -304,25 +232,100 @@ queue_stats(Name, MessagesReady, MessagesUnacknowledge, Messages, Reductions) ->
 queue_deleted(Name) ->
     ets:delete(queue_coarse_metrics, Name),
     %% Delete markers
-    ets_update_element(queue_metrics, Name, {3, 1}),
+    ets:update_element(queue_metrics, Name, {3, 1}),
     CQX = ets:select(channel_queue_exchange_metrics, match_spec_cqx(Name)),
     lists:foreach(fun(Key) ->
-                          ets_update_element(channel_queue_exchange_metrics, Key, {3, 1})
+                          ets:update_element(channel_queue_exchange_metrics, Key, {3, 1})
                   end, CQX),
     CQ = ets:select(channel_queue_metrics, match_spec_cq(Name)),
     lists:foreach(fun(Key) ->
-                          ets_update_element(channel_queue_metrics, Key, {8, 1})
+                          ets:update_element(channel_queue_metrics, Key, {8, 1})
                   end, CQ).
 
+queues_deleted(Queues) ->
+    [ delete_queue_metrics(Queue) || Queue <- Queues ],
+    [
+        begin
+            MatchSpecCondition = build_match_spec_conditions_to_delete_all_queues(QueuesPartition),
+            delete_channel_queue_exchange_metrics(MatchSpecCondition),
+            delete_channel_queue_metrics(MatchSpecCondition)
+        end || QueuesPartition <- partition_queues(Queues)
+    ],
+    ok.
+
+partition_queues(Queues) when length(Queues) >= 1000 ->
+    {Partition, Rest} = lists:split(1000, Queues),
+    [Partition | partition_queues(Rest)];
+partition_queues(Queues) ->
+    [Queues].
+
+delete_queue_metrics(Queue) ->
+    ets:delete(queue_coarse_metrics, Queue),
+    ets:update_element(queue_metrics, Queue, {3, 1}),
+    ok.
+
+delete_channel_queue_exchange_metrics(MatchSpecCondition) ->
+    ChannelQueueExchangeMetricsToUpdate = ets:select(
+        channel_queue_exchange_metrics,
+        [
+            {
+                {{'$2', {'$1', '$3'}}, '_', '_'},
+                [MatchSpecCondition],
+                [{{'$2', {{'$1', '$3'}}}}]
+            }
+        ]
+    ),
+    lists:foreach(fun(Key) ->
+        ets:update_element(channel_queue_exchange_metrics, Key, {3, 1})
+    end, ChannelQueueExchangeMetricsToUpdate).
+
+delete_channel_queue_metrics(MatchSpecCondition) ->
+    ChannelQueueMetricsToUpdate = ets:select(
+        channel_queue_metrics,
+        [
+            {
+                {{'$2', '$1'}, '_', '_', '_', '_', '_', '_', '_'},
+                [MatchSpecCondition],
+                [{{'$2', '$1'}}]
+            }
+        ]
+    ),
+    lists:foreach(fun(Key) ->
+        ets:update_element(channel_queue_metrics, Key, {8, 1})
+    end, ChannelQueueMetricsToUpdate).
+
+% [{'orelse',
+%    {'==', {Queue}, '$1'},
+%    {'orelse',
+%        {'==', {Queue}, '$1'},
+%        % ...
+%            {'orelse',
+%                {'==', {Queue}, '$1'},
+%                {'==', true, true}
+%            }
+%    }
+%  }],
+build_match_spec_conditions_to_delete_all_queues([Queue|Queues]) ->
+     {'orelse',
+         {'==', {Queue}, '$1'},
+         build_match_spec_conditions_to_delete_all_queues(Queues)
+     };
+build_match_spec_conditions_to_delete_all_queues([]) ->
+    true.
+
 node_stats(persister_metrics, Infos) ->
-    ets:insert(node_persister_metrics, {node(), Infos});
+    ets:insert(node_persister_metrics, {node(), Infos}),
+    ok;
 node_stats(coarse_metrics, Infos) ->
-    ets:insert(node_coarse_metrics, {node(), Infos});
+    ets:insert(node_coarse_metrics, {node(), Infos}),
+    ok;
 node_stats(node_metrics, Infos) ->
-    ets:insert(node_metrics, {node(), Infos}).
+    ets:insert(node_metrics, {node(), Infos}),
+    ok.
 
 node_node_stats(Id, Infos) ->
-    ets:insert(node_node_metrics, {Id, Infos}).
+    ets:insert(node_node_metrics, {Id, Infos}),
+    ok.
 
 match_spec_cqx(Id) ->
     [{{{'$2', {'$1', '$3'}}, '_', '_'}, [{'==', {Id}, '$1'}], [{{'$2', {{'$1', '$3'}}}}]}].
@@ -331,10 +334,12 @@ match_spec_cq(Id) ->
     [{{{'$2', '$1'}, '_', '_', '_', '_', '_', '_', '_'}, [{'==', {Id}, '$1'}], [{{'$2', '$1'}}]}].
 
 gen_server2_stats(Pid, BufferLength) ->
-    ets:insert(gen_server2_metrics, {Pid, BufferLength}).
+    ets:insert(gen_server2_metrics, {Pid, BufferLength}),
+    ok.
 
 gen_server2_deleted(Pid) ->
-    ets:delete(gen_server2_metrics, Pid).
+    ets:delete(gen_server2_metrics, Pid),
+    ok.
 
 get_gen_server2_stats(Pid) ->
     case ets:lookup(gen_server2_metrics, Pid) of
