@@ -16,9 +16,16 @@
 
 -module(rabbit_policy_validator).
 
+-behaviour(rabbit_registry_class).
+
+-export([added_to_rabbit_registry/2, removed_from_rabbit_registry/1]).
+
 -export_type([validate_results/0]).
 
 -type(validate_results() ::
         'ok' | {error, string(), [term()]} | [validate_results()]).
 
 -callback validate_policy([{binary(), term()}]) -> validate_results().
+
+added_to_rabbit_registry(_Type, _ModuleName) -> ok.
+removed_from_rabbit_registry(_Type) -> ok.
