@@ -16,6 +16,13 @@
 
 -module(rabbit_queue_master_locator).
 
+-behaviour(rabbit_registry_class).
+
+-export([added_to_rabbit_registry/2, removed_from_rabbit_registry/1]).
+
 -callback description()                -> [proplists:property()].
 -callback queue_master_location(rabbit_types:amqqueue()) ->
     {'ok', node()} | {'error', term()}.
+
+added_to_rabbit_registry(_Type, _ModuleName) -> ok.
+removed_from_rabbit_registry(_Type) -> ok.
