@@ -46,6 +46,7 @@
          send_command_flow/2, send_command_flow/3,
          flush/1]).
 -export([internal_send_command/4, internal_send_command/6]).
+-export([msg_size/1, maybe_gc_large_msg/1]).
 
 %% internal
 -export([enter_mainloop/2, mainloop/2, mainloop1/2]).
@@ -95,7 +96,7 @@
 
 -spec system_code_change(_,_,_,_) -> {'ok',_}.
 -spec system_continue(_,_,#wstate{}) -> any().
--spec system_terminate(_,_,_,_) -> none().
+-spec system_terminate(_,_,_,_) -> no_return().
 
 -spec send_command(pid(), rabbit_framing:amqp_method_record()) -> 'ok'.
 -spec send_command
@@ -126,6 +127,12 @@
          non_neg_integer(), rabbit_types:protocol()) ->
             'ok'.
 
+-spec msg_size
+        (rabbit_types:content() | rabbit_types:message()) -> non_neg_integer().
+
+-spec maybe_gc_large_msg
+        (rabbit_types:content() | rabbit_types:message()) -> non_neg_integer().
+
 %%---------------------------------------------------------------------------
 
 start(Sock, Channel, FrameMax, Protocol, ReaderPid, Identity) ->
@@ -228,15 +235,15 @@ handle_message({'$gen_call', From, flush}, State) ->
     State1;
 handle_message({send_command_and_notify, QPid, ChPid, MethodRecord}, State) ->
     State1 = internal_send_command_async(MethodRecord, State),
-    rabbit_amqqueue:notify_sent(QPid, ChPid),
+    rabbit_amqqueue_common:notify_sent(QPid, ChPid),
     State1;
 handle_message({send_command_and_notify, QPid, ChPid, MethodRecord, Content},
                State) ->
     State1 = internal_send_command_async(MethodRecord, Content, State),
-    rabbit_amqqueue:notify_sent(QPid, ChPid),
+    rabbit_amqqueue_common:notify_sent(QPid, ChPid),
     State1;
 handle_message({'DOWN', _MRef, process, QPid, _Reason}, State) ->
-    rabbit_amqqueue:notify_sent_queue_down(QPid),
+    rabbit_amqqueue_common:notify_sent_queue_down(QPid),
     State;
 handle_message({inet_reply, _, ok}, State) ->
     rabbit_event:ensure_stats_timer(State, #wstate.stats_timer, emit_stats);
@@ -333,7 +340,7 @@ internal_send_command_async(MethodRecord, Content,
                                             pending   = Pending}) ->
     Frames = assemble_frames(Channel, MethodRecord, Content, FrameMax,
                              Protocol),
-    rabbit_basic:maybe_gc_large_msg(Content),
+    maybe_gc_large_msg(Content),
     maybe_flush(State#wstate{pending = [Frames | Pending]}).
 
 %% When the amount of protocol method data buffered exceeds
@@ -380,3 +387,25 @@ port_cmd(Sock, Data) ->
            catch error:Error -> exit({writer, send_failed, Error})
            end,
     ok.
+
+%% Some processes (channel, writer) can get huge amounts of binary
+%% garbage when processing huge messages at high speed (since we only
+%% do enough reductions to GC every few hundred messages, and if each
+%% message is 1MB then that's ugly). So count how many bytes of
+%% message we have processed, and force a GC every so often.
+maybe_gc_large_msg(Content) ->
+    Size = msg_size(Content),
+    Current = case get(msg_size_for_gc) of
+                  undefined -> 0;
+                  C         -> C
+              end,
+    New = Current + Size,
+    put(msg_size_for_gc, case New > 1000000 of
+                             true  -> erlang:garbage_collect(),
+                                      0;
+                             false -> New
+                         end),
+    Size.
+
+msg_size(#content{payload_fragments_rev = PFR}) -> iolist_size(PFR);
+msg_size(#basic_message{content = Content})     -> msg_size(Content).
