@@ -116,15 +116,14 @@
 -define(SETS, sets).
 -define(SET, set).
 
--include("include/old_builtin_types.hrl").
-
 -record(state, {name,
 		strategy               :: strategy(),
 		children = []          :: [child_rec()],
-		dynamics               :: ?DICT_TYPE() | ?SET_TYPE(),
+		dynamics               :: dict:dict() | set:set() | 'undefined',
 		intensity              :: non_neg_integer(),
 		period                 :: pos_integer(),
 		restarts = [],
+		dynamic_restarts = 0   :: non_neg_integer(),
 	        module,
 	        args}).
 -type state() :: #state{}.
@@ -326,7 +325,7 @@ init_children(State, StartSpec) ->
                 {ok, NChildren} ->
                     {ok, State#state{children = NChildren}};
                 {error, NChildren, Reason} ->
-                    terminate_children(NChildren, SupName),
+                    _ = terminate_children(NChildren, SupName),
                     {stop, {shutdown, Reason}}
             end;
         Error ->
@@ -880,7 +879,8 @@ do_restart_delay({RestartType, Delay}, Reason, Child, State) ->
             _TRef = erlang:send_after(trunc(Delay*1000), self(),
                                       {delayed_restart,
                                        {{RestartType, Delay}, Reason, Child}}),
-            {ok, state_del_child(Child, State)}
+            OldPid = Child#child.pid,
+            {ok, replace_child(Child#child{pid=restarting(OldPid)}, State)}
     end.
 
 restart(Child, State) ->
@@ -904,7 +904,7 @@ maybe_restart(Strategy, Child, State) ->
             Id = if ?is_simple(State) -> Child#child.pid;
                     true -> Child#child.name
                  end,
-            timer:apply_after(0,?MODULE,try_again_restart,[self(),Id,Reason]),
+            _ = timer:apply_after(0,?MODULE,try_again_restart,[self(),Id,Reason]),
             {ok,NState2};
         Other ->
             Other
@@ -982,7 +982,7 @@ terminate_children(Children, SupName) ->
 %% we do want them to be shut down as many functions from this module
 %% use this function to just clear everything.
 terminate_children([Child = #child{restart_type=temporary} | Children], SupName, Res) ->
-    do_terminate(Child, SupName),
+    _ = do_terminate(Child, SupName),
     terminate_children(Children, SupName, Res);
 terminate_children([Child | Children], SupName, Res) ->
     NChild = do_terminate(Child, SupName),
@@ -1139,7 +1139,7 @@ wait_dynamic_children(_Child, _Pids, 0, undefined, EStack) ->
 wait_dynamic_children(_Child, _Pids, 0, TRef, EStack) ->
 	%% If the timer has expired before its cancellation, we must empty the
 	%% mail-box of the 'timeout'-message.
-    erlang:cancel_timer(TRef),
+    _ = erlang:cancel_timer(TRef),
     receive
         {timeout, TRef, kill} ->
             EStack
@@ -1447,7 +1447,7 @@ add_restart(State) ->
     I = State#state.intensity,
     P = State#state.period,
     R = State#state.restarts,
-    Now = time_compat:monotonic_time(),
+    Now = erlang:monotonic_time(),
     R1 = add_restart([Now|R], Now, P),
     State1 = State#state{restarts = R1},
     case length(R1) of
@@ -1468,7 +1468,7 @@ add_restart([], _, _) ->
     [].
 
 inPeriod(Time, Now, Period) ->
-    case time_compat:convert_time_unit(Now - Time, native, seconds) of
+    case erlang:convert_time_unit(Now - Time, native, seconds) of
 	T when T > Period ->
 	    false;
 	_ ->
