@@ -36,22 +36,26 @@
 -spec start_link(start_link_args()) -> {'ok', pid(), pid()}.
 
 %%----------------------------------------------------------------------------
-start_link({rabbit_amqp1_0_framing, Sock, Channel, FrameMax, ReaderPid,
-            Username, VHost, Collector}) ->
+start_link({amqp10_framing, Sock, Channel, FrameMax, ReaderPid,
+            Username, VHost, Collector, ProxySocket}) ->
     {ok, SupPid} = supervisor2:start_link(?MODULE, []),
     {ok, WriterPid} =
         supervisor2:start_child(
           SupPid,
           {writer, {rabbit_amqp1_0_writer, start_link,
-                    [Sock, Channel, FrameMax, rabbit_amqp1_0_framing,
+                    [Sock, Channel, FrameMax, amqp10_framing,
                      ReaderPid]},
            intrinsic, ?WORKER_WAIT, worker, [rabbit_amqp1_0_writer]}),
+    SocketForAdapterInfo = case ProxySocket of
+        undefined -> Sock;
+        _         -> ProxySocket
+    end,
     {ok, ChannelPid} =
         supervisor2:start_child(
           SupPid,
           {channel, {rabbit_amqp1_0_session_process, start_link,
                      [{Channel, ReaderPid, WriterPid, Username, VHost, FrameMax,
-                       adapter_info(Sock), Collector}]},
+                       adapter_info(SocketForAdapterInfo), Collector}]},
            intrinsic, ?WORKER_WAIT, worker, [rabbit_amqp1_0_session_process]}),
     {ok, SupPid, ChannelPid}.
 
