@@ -106,7 +106,7 @@ specs(XorQ) ->
 spec(U = #upstream{reconnect_delay = Delay}, #exchange{name = XName}) ->
     {U, {rabbit_federation_exchange_link, start_link, [{U, XName}]},
      {permanent, Delay}, ?WORKER_WAIT, worker,
-     [rabbit_federation_link]};
+     [rabbit_federation_exchange_link]};
 
 spec(Upstream = #upstream{reconnect_delay = Delay}, Q = #amqqueue{}) ->
     {Upstream, {rabbit_federation_queue_link, start_link, [{Upstream, Q}]},
