@@ -21,32 +21,70 @@
 -behaviour(rabbit_mgmt_extension).
 -export([dispatcher/0, web_ui/0]).
 
-static_path(M) ->
-    filename:join(module_path(M), "priv/www").
-
 build_dispatcher(Ignore) ->
-    ThisLocalPath = static_path(?MODULE),
-    LocalPaths = [static_path(M) || M <- modules(Ignore)],
-    cowboy_router:compile([{'_',
-        [{"/api" ++ Path, Mod, Args}
-            || {Path, Mod, Args} <- lists:append([Module:dispatcher()
-            || Module <- modules(Ignore)])]
-     ++ [{"/", cowboy_static, {file, ThisLocalPath ++ "/index.html"}},
-         {"/api", cowboy_static, {file, ThisLocalPath ++ "/api/index.html"}},
-         {"/cli", cowboy_static, {file, ThisLocalPath ++ "/cli/index.html"}},
-         {"/mgmt", rabbit_mgmt_wm_redirect, "/"}]
-     ++ [{"/[...]", rabbit_mgmt_wm_static, LocalPaths}]
-    }]).
+    Routes = build_routes(Ignore),
+    cowboy_router:compile(Routes).
+
+build_routes(Ignore) ->
+    ManagementApp = module_app(?MODULE),
+    Prefix = rabbit_mgmt_util:get_path_prefix(),
+    RootIdxRtes = build_root_index_routes(Prefix, ManagementApp),
+    ApiRdrRte = build_static_index_html_route(Prefix, "/api"),
+    CliRdrRte = build_static_index_html_route(Prefix, "/cli"),
+    MgmtRdrRte = {"/mgmt", rabbit_mgmt_wm_redirect, "/"},
+    LocalPaths = [{module_app(M), "www"} || M <- modules(Ignore)],
+    LocalStaticRte = {"/[...]", rabbit_mgmt_wm_static, LocalPaths},
+    % NB: order is significant in the routing list
+    Routes0 = build_module_routes(Ignore) ++
+        [ApiRdrRte, CliRdrRte, MgmtRdrRte, LocalStaticRte],
+    Routes1 = maybe_add_path_prefix(Routes0, Prefix),
+    % NB: ensure the root routes are first
+    Routes2 = RootIdxRtes ++ Routes1,
+    [{'_', Routes2}].
+
+build_root_index_routes("", ManagementApp) ->
+    [{"/", cowboy_static, root_idx_file(ManagementApp)}];
+build_root_index_routes(Prefix, ManagementApp) ->
+    [{"/", rabbit_mgmt_wm_redirect, Prefix ++ "/"},
+     {Prefix, cowboy_static, root_idx_file(ManagementApp)}].
+
+build_static_index_html_route("", Path) ->
+    {Path, rabbit_mgmt_wm_redirect, Path ++ "/index.html"};
+build_static_index_html_route(Prefix, Path) ->
+    {Path, rabbit_mgmt_wm_redirect, Prefix ++ Path ++ "/index.html"}.
+
+root_idx_file(ManagementApp) ->
+    {priv_file, ManagementApp, "www/index.html"}.
+
+maybe_add_path_prefix(Routes, "") ->
+    Routes;
+maybe_add_path_prefix(Routes, Prefix) ->
+    [{Prefix ++ Path, Mod, Args} || {Path, Mod, Args} <- Routes].
+
+build_module_routes(Ignore) ->
+    Routes = [Module:dispatcher() || Module <- modules(Ignore)],
+    [{"/api" ++ Path, Mod, Args} || {Path, Mod, Args} <- lists:append(Routes)].
 
 modules(IgnoreApps) ->
     [Module || {App, Module, Behaviours} <-
-                   rabbit_misc:all_module_attributes(behaviour),
+               %% Sort rabbitmq_management modules first. This is
+               %% a microoptimization because most files belong to
+               %% this application. Making it first avoids several
+               %% stats(2) which have a great chance of failing in other
+               %% applications.
+               lists:sort(
+                 fun
+                     ({rabbitmq_management, _, _}, _) -> true;
+                     (_, {rabbitmq_management, _, _}) -> false;
+                     ({A, _, _}, {B, _, _})           -> A =< B
+                 end,
+                 rabbit_misc:all_module_attributes(behaviour)),
                not lists:member(App, IgnoreApps),
                lists:member(rabbit_mgmt_extension, Behaviours)].
 
-module_path(Module) ->
-    {file, Here} = code:is_loaded(Module),
-    filename:dirname(filename:dirname(Here)).
+module_app(Module) ->
+    {ok, App} = application:get_application(Module),
+    App.
 
 %%----------------------------------------------------------------------------
 
@@ -76,6 +114,13 @@ dispatcher() ->
      {"/policies",                                             rabbit_mgmt_wm_policies, []},
      {"/policies/:vhost",                                      rabbit_mgmt_wm_policies, []},
      {"/policies/:vhost/:name",                                rabbit_mgmt_wm_policy, []},
+     {"/operator-policies",                                    rabbit_mgmt_wm_operator_policies, []},
+     {"/operator-policies/:vhost",                             rabbit_mgmt_wm_operator_policies, []},
+     {"/operator-policies",                                    rabbit_mgmt_wm_operator_policies, []},
+     {"/operator-policies/:vhost/:name",                       rabbit_mgmt_wm_operator_policy, []},
+     {"/vhost-limits/:vhost/:name",                            rabbit_mgmt_wm_limit, []},
+     {"/vhost-limits",                                         rabbit_mgmt_wm_limits, []},
+     {"/vhost-limits/:vhost",                                  rabbit_mgmt_wm_limits, []},
      {"/connections",                                          rabbit_mgmt_wm_connections, []},
      {"/connections/:connection",                              rabbit_mgmt_wm_connection, []},
      {"/connections/:connection/channels",                     rabbit_mgmt_wm_connection_channels, []},
@@ -102,17 +147,25 @@ dispatcher() ->
      {"/bindings/:vhost/e/:source/:dtype/:destination/:props", rabbit_mgmt_wm_binding, []},
      {"/vhosts",                                               rabbit_mgmt_wm_vhosts, []},
      {"/vhosts/:vhost",                                        rabbit_mgmt_wm_vhost, []},
+     {"/vhosts/:vhost/start/:node",                            rabbit_mgmt_wm_vhost_restart, []},
      {"/vhosts/:vhost/permissions",                            rabbit_mgmt_wm_permissions_vhost, []},
+     {"/vhosts/:vhost/topic-permissions",                      rabbit_mgmt_wm_topic_permissions_vhost, []},
      %% /connections/:connection is already taken, we cannot use our standard scheme here
      {"/vhosts/:vhost/connections",                            rabbit_mgmt_wm_connections_vhost, []},
      %% /channels/:channel is already taken, we cannot use our standard scheme here
      {"/vhosts/:vhost/channels",                               rabbit_mgmt_wm_channels_vhost, []},
-     {"/users",                                                rabbit_mgmt_wm_users, []},
+     {"/users/bulk-delete",                                    rabbit_mgmt_wm_users_bulk_delete, []},
+     {"/users/without-permissions",                            rabbit_mgmt_wm_users, [without_permissions]},
+     {"/users",                                                rabbit_mgmt_wm_users, [all]},
      {"/users/:user",                                          rabbit_mgmt_wm_user, []},
      {"/users/:user/permissions",                              rabbit_mgmt_wm_permissions_user, []},
+     {"/users/:user/topic-permissions",                        rabbit_mgmt_wm_topic_permissions_user, []},
      {"/whoami",                                               rabbit_mgmt_wm_whoami, []},
      {"/permissions",                                          rabbit_mgmt_wm_permissions, []},
      {"/permissions/:vhost/:user",                             rabbit_mgmt_wm_permission, []},
+     {"/topic-permissions",                                    rabbit_mgmt_wm_topic_permissions, []},
+     {"/topic-permissions/:vhost/:user",                       rabbit_mgmt_wm_topic_permission, []},
+     {"/topic-permissions/:vhost/:user/:exchange",             rabbit_mgmt_wm_topic_permission, []},
      {"/aliveness-test/:vhost",                                rabbit_mgmt_wm_aliveness_test, []},
      {"/healthchecks/node",                                    rabbit_mgmt_wm_healthchecks, []},
      {"/healthchecks/node/:node",                              rabbit_mgmt_wm_healthchecks, []},
