@@ -16,6 +16,8 @@
 
 -module(rabbit_mgmt_load_definitions).
 
+-include_lib("rabbit_common/include/rabbit.hrl").
+
 -export([maybe_load_definitions/0]).
 
 %% We want to A) make sure we apply defintions before being open for
@@ -37,7 +39,7 @@ maybe_load_definitions() ->
         none -> ok;
         _    -> case file:read_file(File) of
                     {ok, Body} -> rabbit_log:info(
-                                    "Applying definitions from: ~s~n", [File]),
+                                    "Applying definitions from: ~s", [File]),
                                   load_definitions(Body);
                     {error, E} -> {error, {could_not_read_defs, {File, E}}}
                 end
@@ -45,4 +47,4 @@ maybe_load_definitions() ->
 
 load_definitions(Body) ->
     rabbit_mgmt_wm_definitions:apply_defs(
-      Body, fun () -> ok end, fun (E) -> {error, E} end).
+      Body, ?INTERNAL_USER, fun () -> ok end, fun (E) -> {error, E} end).
