@@ -16,7 +16,7 @@
 
 -module(rabbit_mgmt_wm_channel).
 
--export([init/3, rest_init/2, to_json/2, content_types_provided/2, is_authorized/2]).
+-export([init/2, to_json/2, content_types_provided/2, is_authorized/2]).
 -export([resource_exists/2]).
 -export([variances/2]).
 
@@ -25,10 +25,8 @@
 
 %%--------------------------------------------------------------------
 
-init(_, _, _) -> {upgrade, protocol, cowboy_rest}.
-
-rest_init(Req, _Config) ->
-    {ok, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
+init(Req, _State) ->
+    {cowboy_rest, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
 
 variances(Req, Context) ->
     {[<<"accept-encoding">>, <<"origin">>], Req, Context}.
@@ -46,7 +44,7 @@ to_json(ReqData, Context) ->
     Payload = rabbit_mgmt_format:clean_consumer_details(
                 rabbit_mgmt_format:strip_pids(channel(ReqData))),
     rabbit_mgmt_util:reply(
-      {struct, Payload},
+      maps:from_list(Payload),
       ReqData, Context).
 
 is_authorized(ReqData, Context) ->
