@@ -16,7 +16,7 @@
 
 -module(rabbit_mgmt_wm_cluster_name).
 
--export([init/3, rest_init/2, resource_exists/2, to_json/2,
+-export([init/2, resource_exists/2, to_json/2,
          content_types_provided/2, content_types_accepted/2,
          is_authorized/2, allowed_methods/2, accept_content/2]).
 -export([variances/2]).
@@ -26,10 +26,8 @@
 
 %%--------------------------------------------------------------------
 
-init(_, _, _) -> {upgrade, protocol, cowboy_rest}.
-
-rest_init(Req, _Config) ->
-    {ok, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
+init(Req, _State) ->
+    {cowboy_rest, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
 
 variances(Req, Context) ->
     {[<<"accept-encoding">>, <<"origin">>], Req, Context}.
@@ -50,18 +48,18 @@ to_json(ReqData, Context) ->
     rabbit_mgmt_util:reply(
       [{name, rabbit_nodes:cluster_name()}], ReqData, Context).
 
-accept_content(ReqData0, Context) ->
+accept_content(ReqData0, Context = #context{user = #user{username = Username}}) ->
     rabbit_mgmt_util:with_decode(
       [name], ReqData0, Context, fun([Name], _, ReqData) ->
                                         rabbit_nodes:set_cluster_name(
-                                          as_binary(Name)),
+                                          as_binary(Name), Username),
                                         {true, ReqData, Context}
                                 end).
 
 is_authorized(ReqData, Context) ->
     case cowboy_req:method(ReqData) of
-        {<<"PUT">>, _} -> rabbit_mgmt_util:is_authorized_admin(ReqData, Context);
-        _              -> rabbit_mgmt_util:is_authorized(ReqData, Context)
+        <<"PUT">> -> rabbit_mgmt_util:is_authorized_admin(ReqData, Context);
+        _         -> rabbit_mgmt_util:is_authorized(ReqData, Context)
     end.
 
 as_binary(Val) when is_binary(Val) ->
