@@ -16,7 +16,7 @@
 
 -module(rabbit_mgmt_wm_connection).
 
--export([init/3, rest_init/2, resource_exists/2, to_json/2, content_types_provided/2,
+-export([init/2, resource_exists/2, to_json/2, content_types_provided/2,
          is_authorized/2, allowed_methods/2, delete_resource/2, conn/1]).
 -export([variances/2]).
 
@@ -25,10 +25,8 @@
 
 %%--------------------------------------------------------------------
 
-init(_, _, _) -> {upgrade, protocol, cowboy_rest}.
-
-rest_init(Req, _Config) ->
-    {ok, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
+init(Req, _State) ->
+    {cowboy_rest, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
 
 variances(Req, Context) ->
     {[<<"accept-encoding">>, <<"origin">>], Req, Context}.
@@ -47,18 +45,17 @@ resource_exists(ReqData, Context) ->
 
 to_json(ReqData, Context) ->
     rabbit_mgmt_util:reply(
-      {struct, rabbit_mgmt_format:strip_pids(conn(ReqData))}, ReqData, Context).
+      maps:from_list(rabbit_mgmt_format:strip_pids(conn(ReqData))), ReqData, Context).
 
 delete_resource(ReqData, Context) ->
-    Conn = conn(ReqData),
-    Pid = proplists:get_value(pid, Conn),
-    Reason = case cowboy_req:header(<<"x-reason">>, ReqData) of
-                 {undefined, _} -> "Closed via management plugin";
-                 {V, _}         -> binary_to_list(V)
-             end,
-    case proplists:get_value(type, Conn) of
-        direct  -> amqp_direct_connection:server_close(Pid, 320, Reason);
-        network -> rabbit_networking:close_connection(Pid, Reason)
+    case conn(ReqData) of
+        not_found -> ok;
+        Conn      ->
+            case proplists:get_value(pid, Conn) of
+                undefined -> ok;
+                Pid when is_pid(Pid) ->
+                    force_close_connection(ReqData, Conn, Pid)
+            end
     end,
     {true, ReqData, Context}.
 
@@ -75,3 +72,14 @@ is_authorized(ReqData, Context) ->
 conn(ReqData) ->
     rabbit_mgmt_db:get_connection(rabbit_mgmt_util:id(connection, ReqData),
                                   rabbit_mgmt_util:range_ceil(ReqData)).
+
+force_close_connection(ReqData, Conn, Pid) ->
+    Reason = case cowboy_req:header(<<"x-reason">>, ReqData) of
+                 undefined -> "Closed via management plugin";
+                 V         -> binary_to_list(V)
+             end,
+            case proplists:get_value(type, Conn) of
+                direct  -> amqp_direct_connection:server_close(Pid, 320, Reason);
+                network -> rabbit_networking:close_connection(Pid, Reason)
+            end,
+    ok.
