@@ -18,7 +18,7 @@
 
 %% Lists connections in a vhost
 
--export([init/3, rest_init/2, to_json/2, content_types_provided/2, is_authorized/2,
+-export([init/2, to_json/2, content_types_provided/2, is_authorized/2,
          augmented/2, resource_exists/2]).
 -export([variances/2]).
 
@@ -29,10 +29,8 @@
 
 %%--------------------------------------------------------------------
 
-init(_, _, _) -> {upgrade, protocol, cowboy_rest}.
-
-rest_init(Req, _Config) ->
-    {ok, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
+init(Req, _State) ->
+    {cowboy_rest, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
 
 variances(Req, Context) ->
     {[<<"accept-encoding">>, <<"origin">>], Req, Context}.
