@@ -16,7 +16,7 @@
 
 -module(rabbit_mgmt_wm_node).
 
--export([init/3, rest_init/2, to_json/2, content_types_provided/2, is_authorized/2]).
+-export([init/2, to_json/2, content_types_provided/2, is_authorized/2]).
 -export([resource_exists/2]).
 -export([variances/2]).
 
@@ -25,10 +25,8 @@
 
 %%--------------------------------------------------------------------
 
-init(_, _, _) -> {upgrade, protocol, cowboy_rest}.
-
-rest_init(Req, _Config) ->
-    {ok, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
+init(Req, _State) ->
+    {cowboy_rest, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
 
 variances(Req, Context) ->
     {[<<"accept-encoding">>, <<"origin">>], Req, Context}.
@@ -63,8 +61,8 @@ augment(ReqData, Node, Data) ->
                 Data, [memory, binary]).
 
 augment(Key, ReqData, Node, Data) ->
-    case cowboy_req:qs_val(list_to_binary(atom_to_list(Key)), ReqData) of
-        {<<"true">>, _} -> Res = case rpc:call(Node, rabbit_vm, Key, [], infinity) of
+    case rabbit_mgmt_util:qs_val(list_to_binary(atom_to_list(Key)), ReqData) of
+        <<"true">> -> Res = case rpc:call(Node, rabbit_vm, Key, [], infinity) of
                             {badrpc, _} -> not_available;
                             Result      -> Result
                         end,
