@@ -16,7 +16,7 @@
 
 -module(rabbit_mgmt_wm_parameter).
 
--export([init/3, rest_init/2, resource_exists/2, to_json/2,
+-export([init/2, resource_exists/2, to_json/2,
          content_types_provided/2, content_types_accepted/2,
          is_authorized/2, allowed_methods/2, accept_content/2,
          delete_resource/2]).
@@ -27,10 +27,8 @@
 
 %%--------------------------------------------------------------------
 
-init(_, _, _) -> {upgrade, protocol, cowboy_rest}.
-
-rest_init(Req, _Config) ->
-    {ok, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
+init(Req, _State) ->
+    {cowboy_rest, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
 
 variances(Req, Context) ->
     {[<<"accept-encoding">>, <<"origin">>], Req, Context}.
@@ -65,7 +63,11 @@ accept_content(ReqData0, Context = #context{user = User}) ->
               fun([Value], _, ReqData) ->
                       case rabbit_runtime_parameters:set(
                              VHost, component(ReqData), name(ReqData),
-                             rabbit_misc:json_to_term(Value), User) of
+                             if
+                                is_map(Value) -> maps:to_list(Value);
+                                true -> Value
+                             end,
+                             User) of
                           ok ->
                               {true, ReqData, Context};
                           {error_string, Reason} ->
@@ -77,9 +79,9 @@ accept_content(ReqData0, Context = #context{user = User}) ->
               end)
     end.
 
-delete_resource(ReqData, Context) ->
+delete_resource(ReqData, Context = #context{user = #user{username = Username}}) ->
     ok = rabbit_runtime_parameters:clear(
-           rabbit_mgmt_util:vhost(ReqData), component(ReqData), name(ReqData)),
+           rabbit_mgmt_util:vhost(ReqData), component(ReqData), name(ReqData), Username),
     {true, ReqData, Context}.
 
 is_authorized(ReqData, Context) ->
