@@ -16,7 +16,7 @@
 
 -module(rabbit_mgmt_wm_parameters).
 
--export([init/3, rest_init/2, to_json/2, content_types_provided/2, is_authorized/2,
+-export([init/2, to_json/2, content_types_provided/2, is_authorized/2,
          resource_exists/2, basic/1]).
 -export([fix_shovel_publish_properties/1]).
 -export([variances/2]).
@@ -26,10 +26,8 @@
 
 %%--------------------------------------------------------------------
 
-init(_, _, _) -> {upgrade, protocol, cowboy_rest}.
-
-rest_init(Req, _Config) ->
-    {ok, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
+init(Req, _State) ->
+    {cowboy_rest, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
 
 variances(Req, Context) ->
     {[<<"accept-encoding">>, <<"origin">>], Req, Context}.
@@ -53,22 +51,6 @@ is_authorized(ReqData, Context) ->
 
 %%--------------------------------------------------------------------
 
-basic(ReqData) ->
-    Raw = case rabbit_mgmt_util:id(component, ReqData) of
-              none -> rabbit_runtime_parameters:list();
-              Name -> case rabbit_mgmt_util:vhost(ReqData) of
-                          none      -> rabbit_runtime_parameters:list_component(
-                                         Name);
-                          not_found -> not_found;
-                          VHost     -> rabbit_runtime_parameters:list(
-                                         VHost, Name)
-                      end
-          end,
-    case Raw of
-        not_found -> not_found;
-        _         -> [rabbit_mgmt_format:parameter(fix_shovel_publish_properties(P)) || P <- Raw]
-    end.
-
 %% Hackish fix to make sure we return a JSON object instead of an empty list
 %% when the publish-properties value is empty. Should be removed in 3.7.0
 %% when we switch to a new JSON library.
@@ -87,3 +69,19 @@ fix_shovel_publish_properties(P) ->
             end;
         _ -> P
     end.
+
+basic(ReqData) ->
+    Raw = case rabbit_mgmt_util:id(component, ReqData) of
+              none -> rabbit_runtime_parameters:list();
+              Name -> case rabbit_mgmt_util:vhost(ReqData) of
+                          none      -> rabbit_runtime_parameters:list_component(
+                                         Name);
+                          not_found -> not_found;
+                          VHost     -> rabbit_runtime_parameters:list(
+                                         VHost, Name)
+                      end
+          end,
+    case Raw of
+        not_found -> not_found;
+        _         -> [rabbit_mgmt_format:parameter(fix_shovel_publish_properties(P)) || P <- Raw]
+    end.
