@@ -16,6 +16,10 @@
 
 -module(rabbit_mgmt_external_stats).
 
+%% Transitional step until we can require Erlang/OTP 21 and
+%% use the now recommended try/catch syntax for obtaining the stack trace.
+-compile(nowarn_deprecated_function).
+
 -behaviour(gen_server).
 
 -export([start_link/0]).
@@ -36,8 +40,9 @@
 -define(OTHER_KEYS, [name, partitions, os_pid, fd_total, sockets_total, mem_limit,
                      mem_alarm, disk_free_limit, disk_free_alarm, proc_total,
                      rates_mode, uptime, run_queue, processors, exchange_types,
-                     auth_mechanisms, applications, contexts, log_file,
-                     sasl_log_file, db_dir, config_files, net_ticktime, enabled_plugins]).
+                     auth_mechanisms, applications, contexts, log_files,
+                     db_dir, config_files, net_ticktime, enabled_plugins,
+                     mem_calculation_strategy]).
 
 %%--------------------------------------------------------------------
 
@@ -182,7 +187,9 @@ i(sockets_used,    _State) ->
 i(sockets_total,   _State) ->
     proplists:get_value(sockets_limit, file_handle_cache:info([sockets_limit]));
 i(os_pid,          _State) -> list_to_binary(os:getpid());
-i(mem_used,        _State) -> erlang:memory(total);
+
+i(mem_used,        _State) -> vm_memory_monitor:get_process_memory();
+i(mem_calculation_strategy, _State) -> vm_memory_monitor:get_memory_calculation_strategy();
 i(mem_limit,       _State) -> vm_memory_monitor:get_memory_limit();
 i(mem_alarm,       _State) -> resource_alarm_set(memory);
 i(proc_used,       _State) -> erlang:system_info(process_count);
@@ -197,8 +204,7 @@ i(uptime,          _State) -> {Total, _} = erlang:statistics(wall_clock),
                                 Total;
 i(rates_mode,      _State) -> rabbit_mgmt_db_handler:rates_mode();
 i(exchange_types,  _State) -> list_registry_plugins(exchange);
-i(log_file,        _State) -> log_location(kernel);
-i(sasl_log_file,   _State) -> log_location(sasl);
+i(log_files,       _State) -> [list_to_binary(F) || F <- rabbit:log_locations()];
 i(db_dir,          _State) -> list_to_binary(rabbit_mnesia:dir());
 i(config_files,    _State) -> [list_to_binary(F) || F <- rabbit:config_files()];
 i(net_ticktime,    _State) -> net_kernel:get_net_ticktime();
@@ -224,12 +230,6 @@ i(context_switches, _State) ->
     {Sw, 0} = erlang:statistics(context_switches),
     Sw.
 
-log_location(Type) ->
-    case rabbit:log_location(Type) of
-        tty  -> <<"tty">>;
-        File -> list_to_binary(File)
-    end.
-
 resource_alarm_set(Source) ->
     lists:member({{resource_limit, Source, node()},[]},
                  rabbit_alarm:get_alarms()).
