@@ -44,33 +44,33 @@ list() ->
 %%----------------------------------------------------------------------------
 
 init([]) ->
-    {ok, #state{client_ids = dict:new()}}. % clientid -> {pid, monitor}
+    {ok, #state{client_ids = #{}}}. % clientid -> {pid, monitor}
 
 %%--------------------------------------------------------------------------
 
 handle_call({register, ClientId, Pid}, _From,
             State = #state{client_ids = Ids}) ->
-    Ids1 = case dict:find(ClientId, Ids) of
+    Ids1 = case maps:find(ClientId, Ids) of
                {ok, {OldPid, MRef}} when Pid =/= OldPid ->
                    catch gen_server2:cast(OldPid, duplicate_id),
                    erlang:demonitor(MRef),
-                   dict:erase(ClientId, Ids);
+                   maps:remove(ClientId, Ids);
                error ->
                    Ids
            end,
-    Ids2 = dict:store(ClientId, {Pid, erlang:monitor(process, Pid)}, Ids1),
+    Ids2 = maps:put(ClientId, {Pid, erlang:monitor(process, Pid)}, Ids1),
     {reply, ok, State#state{client_ids = Ids2}};
 
 handle_call({unregister, ClientId, Pid}, _From, State = #state{client_ids = Ids}) ->
-    {Reply, Ids1} = case dict:find(ClientId, Ids) of
+    {Reply, Ids1} = case maps:find(ClientId, Ids) of
                         {ok, {Pid, MRef}} -> erlang:demonitor(MRef),
-                                             {ok, dict:erase(ClientId, Ids)};
+                                             {ok, maps:remove(ClientId, Ids)};
                         _                 -> {ok, Ids}
                     end,
     {reply, Reply, State#state{ client_ids = Ids1 }};
 
 handle_call(list, _From, State = #state{client_ids = Ids}) ->
-    {reply, dict:to_list(Ids), State};
+    {reply, maps:to_list(Ids), State};
 
 handle_call(Msg, _From, State) ->
     {stop, {unhandled_call, Msg}, State}.
@@ -83,10 +83,11 @@ handle_info({'EXIT', _, {shutdown, closed}}, State) ->
 
 handle_info({'DOWN', MRef, process, DownPid, _Reason},
             State = #state{client_ids = Ids}) ->
-    Ids1 = dict:filter(fun (ClientId, {Pid, M})
+    Ids1 = maps:filter(fun (ClientId, {Pid, M})
                            when Pid =:= DownPid, MRef =:= M ->
-                               rabbit_log:warning("MQTT disconnect from ~p~n",
-                                                  [ClientId]),
+                               rabbit_log_connection:warning(
+                                              "MQTT disconnect from ~p~n",
+                                              [ClientId]),
                                false;
                            (_, _) ->
                                true
