@@ -15,6 +15,11 @@
 %%
 
 -module(rabbit_mqtt_reader).
+
+%% Transitional step until we can require Erlang/OTP 21 and
+%% use the now recommended try/catch syntax for obtaining the stack trace.
+-compile(nowarn_deprecated_function).
+
 -behaviour(gen_server2).
 
 -export([start_link/3]).
@@ -38,15 +43,6 @@ start_link(KeepaliveSup, Ref, Sock) ->
     Pid = proc_lib:spawn_link(?MODULE, init,
                               [[KeepaliveSup, Ref, Sock]]),
 
-    %% In the event that somebody floods us with connections, the
-    %% reader processes can spew log events at error_logger faster
-    %% than it can keep up, causing its mailbox to grow unbounded
-    %% until we eat all the memory available and crash. So here is a
-    %% meaningless synchronous call to the underlying gen_event
-    %% mechanism. When it returns the mailbox is drained, and we
-    %% return to our caller to accept more connections.
-    gen_event:which_handlers(error_logger),
-
     {ok, Pid}.
 
 conserve_resources(Pid, _, {_, Conserve, _}) ->
@@ -63,17 +59,18 @@ info(Pid, InfoItems) ->
 
 init([KeepaliveSup, Ref, Sock]) ->
     process_flag(trap_exit, true),
-    rabbit_net:accept_ack(Ref, Sock),
+    RealSocket = rabbit_net:unwrap_socket(Sock),
+    rabbit_networking:accept_ack(Ref, RealSocket),
     case rabbit_net:connection_string(Sock, inbound) of
         {ok, ConnStr} ->
-            log(debug, "MQTT accepting TCP connection ~p (~s)~n", [self(), ConnStr]),
+            rabbit_log_connection:debug("MQTT accepting TCP connection ~p (~s)~n", [self(), ConnStr]),
             rabbit_alarm:register(
               self(), {?MODULE, conserve_resources, []}),
-            ProcessorState = rabbit_mqtt_processor:initial_state(Sock,ssl_login_name(Sock)),
+            ProcessorState = rabbit_mqtt_processor:initial_state(Sock,ssl_login_name(RealSocket)),
             gen_server2:enter_loop(?MODULE, [],
              rabbit_event:init_stats_timer(
               control_throttle(
-               #state{socket                 = Sock,
+               #state{socket                 = RealSocket,
                       conn_name              = ConnStr,
                       await_recv             = false,
                       connection_state       = running,
@@ -85,13 +82,13 @@ init([KeepaliveSup, Ref, Sock]) ->
                       proc_state             = ProcessorState }), #state.stats_timer),
              {backoff, 1000, 1000, 10000});
         {network_error, Reason} ->
-            rabbit_net:fast_close(Sock),
+            rabbit_net:fast_close(RealSocket),
             terminate({shutdown, Reason}, undefined);
         {error, enotconn} ->
-            rabbit_net:fast_close(Sock),
+            rabbit_net:fast_close(RealSocket),
             terminate(shutdown, undefined);
         {error, Reason} ->
-            rabbit_net:fast_close(Sock),
+            rabbit_net:fast_close(RealSocket),
             terminate({network_error, Reason}, undefined)
     end.
 
@@ -109,7 +106,7 @@ handle_call(Msg, From, State) ->
 handle_cast(duplicate_id,
             State = #state{ proc_state = PState,
                             conn_name  = ConnName }) ->
-    log(warning, "MQTT disconnecting duplicate client id ~p (~p)~n",
+    rabbit_log_connection:warning("MQTT disconnecting duplicate client id ~p (~p)~n",
                  [rabbit_mqtt_processor:info(client_id, PState), ConnName]),
     {stop, {shutdown, duplicate_id}, State};
 
@@ -171,7 +168,7 @@ handle_info({start_keepalives, Keepalive},
 
 handle_info(keepalive_timeout, State = #state {conn_name = ConnStr,
                                                proc_state = PState}) ->
-    log(error, "closing MQTT connection ~p (keepalive timeout)~n", [ConnStr]),
+    rabbit_log_connection:error("closing MQTT connection ~p (keepalive timeout)~n", [ConnStr]),
     send_will_and_terminate(PState, {shutdown, keepalive_timeout}, State);
 
 handle_info(emit_stats, State) ->
@@ -185,42 +182,42 @@ terminate(Reason, State) ->
     do_terminate(Reason, State).
 
 do_terminate({network_error, {ssl_upgrade_error, closed}, ConnStr}, _State) ->
-    log(error, "MQTT detected TLS upgrade error on ~s: connection closed~n",
+    rabbit_log_connection:error("MQTT detected TLS upgrade error on ~s: connection closed~n",
        [ConnStr]);
 
 do_terminate({network_error,
            {ssl_upgrade_error,
             {tls_alert, "handshake failure"}}, ConnStr}, _State) ->
-    log(error, "MQTT detected TLS upgrade error on ~s: handshake failure~n",
+    rabbit_log_connection:error("MQTT detected TLS upgrade error on ~s: handshake failure~n",
        [ConnStr]);
 
 do_terminate({network_error,
            {ssl_upgrade_error,
             {tls_alert, "unknown ca"}}, ConnStr}, _State) ->
-    log(error, "MQTT detected TLS certificate verification error on ~s: alert 'unknown CA'~n",
+    rabbit_log_connection:error("MQTT detected TLS certificate verification error on ~s: alert 'unknown CA'~n",
        [ConnStr]);
 
 do_terminate({network_error,
            {ssl_upgrade_error,
             {tls_alert, Alert}}, ConnStr}, _State) ->
-    log(error, "MQTT detected TLS upgrade error on ~s: alert ~s~n",
+    rabbit_log_connection:error("MQTT detected TLS upgrade error on ~s: alert ~s~n",
        [ConnStr, Alert]);
 
 do_terminate({network_error, {ssl_upgrade_error, Reason}, ConnStr}, _State) ->
-    log(error, "MQTT detected TLS upgrade error on ~s: ~p~n",
+    rabbit_log_connection:error("MQTT detected TLS upgrade error on ~s: ~p~n",
         [ConnStr, Reason]);
 
 do_terminate({network_error, Reason, ConnStr}, _State) ->
-    log(error, "MQTT detected network error on ~s: ~p~n",
+    rabbit_log_connection:error("MQTT detected network error on ~s: ~p~n",
         [ConnStr, Reason]);
 
 do_terminate({network_error, Reason}, _State) ->
-    log(error, "MQTT detected network error: ~p~n", [Reason]);
+    rabbit_log_connection:error("MQTT detected network error: ~p~n", [Reason]);
 
 do_terminate(normal, #state{proc_state = ProcState,
                          conn_name  = ConnName}) ->
     rabbit_mqtt_processor:close_connection(ProcState),
-    log(info, "closing MQTT connection ~p (~s)~n", [self(), ConnName]),
+    rabbit_log_connection:info("closing MQTT connection ~p (~s)~n", [self(), ConnName]),
     ok;
 
 do_terminate(_Reason, #state{proc_state = ProcState}) ->
@@ -244,7 +241,7 @@ ssl_login_name(Sock) ->
 %%----------------------------------------------------------------------------
 
 log_new_connection(#state{conn_name = ConnStr}) ->
-    log(info, "accepting MQTT connection ~p (~s)~n", [self(), ConnStr]).
+    rabbit_log_connection:info("accepting MQTT connection ~p (~s)~n", [self(), ConnStr]).
 
 process_received_bytes(<<>>, State = #state{proc_state = ProcState,
                                             received_connect_frame = false}) ->
@@ -275,22 +272,24 @@ process_received_bytes(Bytes,
                                     proc_state = ProcState1,
                                     connection = ConnPid });
                 {error, Reason, ProcState1} ->
-                    log(info, "MQTT protocol error ~p for connection ~s~n",
+                    rabbit_log_connection:info("MQTT protocol error ~p for connection ~s~n",
                         [Reason, ConnStr]),
                     {stop, {shutdown, Reason}, pstate(State, ProcState1)};
                 {error, Error} ->
-                    log(error, "MQTT detected framing error '~p' for connection ~s~n",
+                    rabbit_log_connection:error("MQTT detected framing error '~p' for connection ~s~n",
                         [Error, ConnStr]),
                     {stop, {shutdown, Error}, State};
                 {stop, ProcState1} ->
-                    {stop, normal, pstate(State, ProcState1)}
+                    {stop, normal, pstate(State, ProcState1)};
+                {err, unauthorized = Reason, ProcState1} ->
+                    {stop, {shutdown, Reason}, pstate(State, ProcState1)}
             end;
         {error, {cannot_parse, Error, Stacktrace}} ->
-            log(error, "MQTT cannot parse frame for connection '~s', unparseable payload: ~p, error: {~p, ~p} ~n",
+            rabbit_log_connection:error("MQTT cannot parse frame for connection '~s', unparseable payload: ~p, error: {~p, ~p} ~n",
                 [ConnStr, Bytes, Error, Stacktrace]),
             {stop, {shutdown, Error}, State};
         {error, Error} ->
-            log(error, "MQTT detected framing error '~p' for connection ~s~n",
+            rabbit_log_connection:error("MQTT detected framing error '~p' for connection ~s~n",
                 [ConnStr, Error]),
             {stop, {shutdown, Error}, State}
     end.
@@ -315,8 +314,6 @@ parse(Bytes, ParseState) ->
             {error, {cannot_parse, Reason, erlang:get_stacktrace()}}
     end.
 
-log(Level, Fmt, Args) -> rabbit_log:log(connection, Level, Fmt, Args).
-
 send_will_and_terminate(PState, State) ->
     send_will_and_terminate(PState, {shutdown, conn_closed}, State).
 
@@ -329,18 +326,18 @@ network_error(closed,
               State = #state{ conn_name  = ConnStr,
                               proc_state = PState }) ->
     MqttConn = PState#proc_state.connection,
-    log(case MqttConn of
-            undefined  -> debug;
-            _          -> info
-        end,
-        "MQTT detected network error for ~p: peer closed TCP connection~n",
-        [ConnStr]),
+    Fmt = "MQTT detected network error for ~p: peer closed TCP connection~n",
+    Args = [ConnStr],
+    case MqttConn of
+        undefined  -> rabbit_log_connection:debug(Fmt, Args);
+        _          -> rabbit_log_connection:info(Fmt, Args)
+    end,
     send_will_and_terminate(PState, State);
 
 network_error(Reason,
               State = #state{ conn_name  = ConnStr,
                               proc_state = PState }) ->
-    log(info, "MQTT detected network error for ~p: ~p~n", [ConnStr, Reason]),
+    rabbit_log_connection:info("MQTT detected network error for ~p: ~p~n", [ConnStr, Reason]),
     send_will_and_terminate(PState, State).
 
 run_socket(State = #state{ connection_state = blocked }) ->
@@ -415,7 +412,7 @@ info_internal(reductions, _State) ->
     {reductions, Reductions} = erlang:process_info(self(), reductions),
     Reductions;
 info_internal(conn_name, #state{conn_name = Val}) ->
-    Val;
+    rabbit_data_coercion:to_binary(Val);
 info_internal(connection_state, #state{received_connect_frame = false}) ->
     starting;
 info_internal(connection_state, #state{connection_state = Val}) ->
