@@ -18,10 +18,13 @@
 
 -export([start/0, stop/0, start/2, stop/1]).
 
-start()           -> rabbit_shovel_sup:start_link(), ok.
+start() ->
+    _ = rabbit_shovel_sup:start_link(),
+    ok.
 
-stop()            -> ok.
+stop() -> ok.
 
-start(normal, []) -> rabbit_shovel_sup:start_link().
+start(normal, []) ->
+    rabbit_shovel_sup:start_link().
 
-stop(_State)      -> ok.
+stop(_State) -> ok.
