@@ -16,8 +16,7 @@
 
 -module(rabbit_top_wm_process).
 
--export([init/3]).
--export([rest_init/2, to_json/2, resource_exists/2, content_types_provided/2,
+-export([init/2, to_json/2, resource_exists/2, content_types_provided/2,
          is_authorized/2]).
 
 -define(ADDITIONAL_INFO,
@@ -28,9 +27,9 @@
 
 %%--------------------------------------------------------------------
 
-init(_, _, _) -> {upgrade, protocol, cowboy_rest}.
+init(Req, _State) ->
+    {cowboy_rest, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
 
-rest_init(ReqData, _) -> {ok, ReqData, #context{}}.
 
 content_types_provided(ReqData, Context) ->
    {[{<<"application/json">>, to_json}], ReqData, Context}.
