@@ -15,8 +15,7 @@
 
 -module(rabbit_tracing_wm_trace).
 
--export([init/3]).
--export([rest_init/2, resource_exists/2, to_json/2,
+-export([init/2, resource_exists/2, to_json/2,
          content_types_provided/2, content_types_accepted/2,
          is_authorized/2, allowed_methods/2, accept_content/2,
          delete_resource/2]).
@@ -29,9 +28,9 @@
 -include_lib("rabbitmq_management_agent/include/rabbit_mgmt_records.hrl").
 
 %%--------------------------------------------------------------------
-init(_, _, _) -> {upgrade, protocol, cowboy_rest}.
+init(Req, _State) ->
+    {cowboy_rest, rabbit_mgmt_cors:set_headers(Req, ?MODULE), #context{}}.
 
-rest_init(ReqData, _) -> {ok, ReqData, #context{}}.
 
 content_types_provided(ReqData, Context) ->
    {[{<<"application/json">>, to_json}], ReqData, Context}.
@@ -43,13 +42,13 @@ allowed_methods(ReqData, Context) ->
     {[<<"HEAD">>, <<"GET">>, <<"PUT">>, <<"DELETE">>], ReqData, Context}.
 
 resource_exists(ReqData, Context) ->
-    {case trace(ReqData) of
+    {case trace(ReqData, Context) of
          not_found -> false;
          _         -> true
      end, ReqData, Context}.
 
 to_json(ReqData, Context) ->
-    rabbit_mgmt_util:reply(trace(ReqData), ReqData, Context).
+    rabbit_mgmt_util:reply(trace(ReqData, Context), ReqData, Context).
 
 accept_content(ReqData0, Ctx) ->
     case rabbit_mgmt_util:vhost(ReqData0) of
@@ -60,9 +59,10 @@ accept_content(ReqData0, Ctx) ->
             rabbit_mgmt_util:with_decode(
               [format, pattern], ReqData0, Ctx,
               fun([_, _], Trace, ReqData) ->
-                      Fs = [fun val_payload_bytes/3, fun val_format/3,
-                            fun val_create/3],
-                      case lists:foldl(fun (F,  ok)  -> F(VHost, Name, Trace);
+                      Fs = [fun val_payload_bytes/5, fun val_format/5,
+                            fun val_create/5],
+                      case lists:foldl(fun (F,  ok)  -> F(ReqData, Ctx, VHost,
+                                                          Name, Trace);
                                            (_F, Err) -> Err
                                        end, ok, Fs) of
                           ok  -> {true, ReqData, Ctx};
@@ -75,8 +75,8 @@ accept_content(ReqData0, Ctx) ->
 
 delete_resource(ReqData, Context) ->
     VHost = rabbit_mgmt_util:vhost(ReqData),
-    Name = rabbit_mgmt_util:id(name, ReqData),
-    ok = rabbit_tracing_traces:stop(VHost, Name),
+    rabbit_tracing_util:apply_on_node(ReqData, Context, rabbit_tracing_traces, stop,
+                                      [VHost, rabbit_mgmt_util:id(name, ReqData)]),
     {true, ReqData, Context}.
 
 is_authorized(ReqData, Context) ->
@@ -84,27 +84,31 @@ is_authorized(ReqData, Context) ->
 
 %%--------------------------------------------------------------------
 
-trace(ReqData) ->
+trace(ReqData, Context) ->
     case rabbit_mgmt_util:vhost(ReqData) of
         not_found -> not_found;
-        VHost     -> rabbit_tracing_traces:lookup(
-                       VHost, rabbit_mgmt_util:id(name, ReqData))
+        VHost     ->
+            Name = rabbit_mgmt_util:id(name, ReqData),
+            rabbit_tracing_util:apply_on_node(ReqData, Context, rabbit_tracing_traces,
+                                              lookup, [VHost, Name])
     end.
 
-val_payload_bytes(_VHost, _Name, Trace) ->
-    case is_integer(pget(max_payload_bytes, Trace, 0)) of
+val_payload_bytes(_ReqData, _Context, _VHost, _Name, Trace) ->
+    case is_integer(maps:get(max_payload_bytes, Trace, 0)) of
         false -> <<"max_payload_bytes not integer">>;
         true  -> ok
     end.
 
-val_format(_VHost, _Name, Trace) ->
-    case lists:member(pget(format, Trace), [<<"json">>, <<"text">>]) of
+val_format(_ReqData, _Context, _VHost, _Name, Trace) ->
+    case lists:member(maps:get(format, Trace), [<<"json">>, <<"text">>]) of
         false -> <<"format not json or text">>;
         true  -> ok
     end.
 
-val_create(VHost, Name, Trace) ->
-    case rabbit_tracing_traces:create(VHost, Name, Trace) of
+val_create(ReqData, Context, VHost, Name, Trace) ->
+    case rabbit_tracing_util:apply_on_node(
+           ReqData, Context, rabbit_tracing_traces, create,
+           [VHost, Name, maps:to_list(Trace)]) of
         {ok, _} -> ok;
         _       -> ?ERR
     end.
