@@ -37,13 +37,20 @@ ensure_listener(Listener) ->
             {error, {no_port_given, Listener}};
         _ ->
             {Transport, TransportOpts, ProtoOpts} = preprocess_config(Listener),
+            ProtoOptsMap = maps:from_list(ProtoOpts),
+            StreamHandlers = stream_handlers_config(ProtoOpts),
+            rabbit_log:debug("Starting HTTP[S] listener with transport ~s, options ~p and protocol options ~p, stream handlers ~p",
+                             [Transport, TransportOpts, ProtoOptsMap, StreamHandlers]),
+            CowboyOptsMap =
+                maps:merge(#{env =>
+                                #{rabbit_listener => Listener},
+                             middlewares =>
+                                [rabbit_cowboy_middleware, cowboy_router, cowboy_handler],
+                             stream_handlers => StreamHandlers},
+                           ProtoOptsMap),
             Child = ranch:child_spec(name(Listener), 100,
                 Transport, TransportOpts,
-                cowboy_protocol, [
-                    {env, [{rabbit_listener, Listener}]},
-                    {middlewares, [rabbit_cowboy_middleware, cowboy_router, cowboy_handler]},
-                    {onresponse, fun rabbit_cowboy_middleware:onresponse/4}
-                    | ProtoOpts]),
+                cowboy_clear, CowboyOptsMap),
             case supervisor:start_child(?SUP, Child) of
                 {ok,                      _}  -> new;
                 {error, {already_started, _}} -> existing;
@@ -115,11 +122,15 @@ transport_config(Options0) ->
     end.
 
 protocol_config(Options) ->
-    ProtoOpts = proplists:get_value(cowboy_opts, Options, []),
-    %% Compress responses by default.
-    case lists:keyfind(compress, 1, ProtoOpts) of
-        false -> [{compress, true}|ProtoOpts];
-        _ -> ProtoOpts
+    proplists:get_value(cowboy_opts, Options, []).
+
+stream_handlers_config(Options) ->
+    case lists:keyfind(compress, 1, Options) of
+        {compress, false} -> [rabbit_cowboy_stream_h, cowboy_stream_h];
+        %% Compress by default. Since 2.0 the compress option in cowboys
+        %% has been replaced by the cowboy_compress_h handler
+        %% Compress is not applied if data < 300 bytes
+        _ -> [rabbit_cowboy_stream_h, cowboy_compress_h, cowboy_stream_h]
     end.
 
 check_error(Listener, Error) ->
