@@ -21,7 +21,7 @@
 -include_lib("amqp_client/include/amqp_client.hrl").
 
 -export([start_link/1]).
--export([sockjs_msg/2, sockjs_closed/1]).
+-export([msg/2, closed/1]).
 
 -export([init/1, handle_call/3, handle_info/2, terminate/2,
          code_change/3, handle_cast/2]).
@@ -33,15 +33,15 @@
 start_link(Params) ->
     gen_server:start_link(?MODULE, Params, []).
 
-sockjs_msg(Pid, Data) ->
-    gen_server:cast(Pid, {sockjs_msg, Data}).
+msg(Pid, Data) ->
+    gen_server:cast(Pid, {msg, Data}).
 
-sockjs_closed(Pid) ->
-    gen_server:cast(Pid, sockjs_closed).
+closed(Pid) ->
+    gen_server:cast(Pid, closed).
 
 %%----------------------------------------------------------------------------
 
-init({SupPid, Conn, Heartbeat, Conn}) ->
+init({SupPid, Conn, Heartbeat}) ->
     ok = file_handle_cache:obtain(),
     process_flag(trap_exit, true),
     {ok, ProcessorState} = init_processor_state(Conn),
@@ -54,12 +54,12 @@ init({SupPid, Conn, Heartbeat, Conn}) ->
                   heartbeat_mode = Heartbeat},
            #state.stats_timer)}.
 
-init_processor_state(Conn) ->
+init_processor_state({ConnMod, ConnProps}) ->
     SendFun = fun (_Sync, Data) ->
-                      Conn:send(Data),
+                      ConnMod:send(ConnProps, Data),
                       ok
               end,
-    Info = Conn:info(),
+    Info = ConnMod:info(ConnProps),
     Headers = proplists:get_value(headers, Info),
 
     UseHTTPAuth = application:get_env(rabbitmq_web_stomp, use_http_auth, false),
@@ -75,9 +75,8 @@ init_processor_state(Conn) ->
                     StompConfig1 = rabbit_stomp:parse_default_user(UserConfig, StompConfig0),
                     StompConfig1#stomp_configuration{force_default_creds = true};
                 {_, AuthHd} ->
-                    {<<"basic">>, {HTTPLogin, HTTPPassCode}}
-                        = cowboy_http:token_ci(list_to_binary(AuthHd),
-                                               fun cowboy_http:authorization/2),
+                    {basic, HTTPLogin, HTTPPassCode}
+                        = cow_http_hd:parse_authorization(rabbit_data_coercion:to_binary(AuthHd)),
                     StompConfig0#stomp_configuration{
                       default_login = HTTPLogin,
                       default_passcode = HTTPPassCode,
@@ -100,7 +99,7 @@ init_processor_state(Conn) ->
         {SendFun, AdapterInfo, none, PeerAddr}),
     {ok, ProcessorState}.
 
-handle_cast({sockjs_msg, Data}, State = #state{proc_state  = ProcessorState,
+handle_cast({msg, Data}, State = #state{proc_state  = ProcessorState,
                                                parse_state = ParseState,
                                                connection  = ConnPid}) ->
     case process_received_bytes(Data, ProcessorState, ParseState, ConnPid) of
@@ -115,7 +114,7 @@ handle_cast({sockjs_msg, Data}, State = #state{proc_state  = ProcessorState,
                                 proc_state  = NewProcState}}
     end;
 
-handle_cast(sockjs_closed, State) ->
+handle_cast(closed, State) ->
     {stop, normal, State};
 
 handle_cast(client_timeout, State) ->
@@ -167,13 +166,13 @@ handle_info({start_heartbeats, _},
 handle_info({start_heartbeats, {0, 0}}, State) ->
     {noreply, State};
 handle_info({start_heartbeats, {SendTimeout, ReceiveTimeout}},
-            State = #state{conn = Conn,
+            State = #state{conn = {ConnMod, ConnProps},
                            heartbeat_sup = SupPid,
                            heartbeat_mode = heartbeat}) ->
-    Info = Conn:info(),
+    Info = ConnMod:info(ConnProps),
     Sock = proplists:get_value(socket, Info),
     Pid = self(),
-    SendFun = fun () -> Conn:send(<<$\n>>), ok end,
+    SendFun = fun () -> ConnMod:send(ConnProps, <<$\n>>), ok end,
     ReceiveFun = fun() -> gen_server2:cast(Pid, client_timeout) end,
     Heartbeat = rabbit_heartbeat:start(SupPid, Sock, SendTimeout,
                                        SendFun, ReceiveTimeout, ReceiveFun),
@@ -203,11 +202,12 @@ handle_info(Info, State) ->
 handle_call(Request, _From, State) ->
     {stop, {odd_request, Request}, State}.
 
-terminate(_Reason, State = #state{conn = Conn, proc_state = ProcessorState}) ->
+terminate(_Reason, State = #state{conn = {ConnMod, ConnProps},
+                                  proc_state = ProcessorState}) ->
     maybe_emit_stats(State),
     ok = file_handle_cache:release(),
     rabbit_stomp_processor:flush_and_die(ProcessorState),
-    Conn:close(1000, "STOMP died"),
+    ConnMod:close(ConnProps, 1000, "STOMP died"),
     ok.
 
 code_change(_OldVsn, State, _Extra) ->
@@ -249,8 +249,8 @@ emit_stats(State=#state{connection = C}) when C == none; C == undefined ->
     %% established, as this causes orphan entries on the stats database
     State1 = rabbit_event:reset_stats_timer(State, #state.stats_timer),
     State1;
-emit_stats(State=#state{conn=Conn, connection=ConnPid}) ->
-    Info = Conn:info(),
+emit_stats(State=#state{conn={ConnMod, ConnProps}, connection=ConnPid}) ->
+    Info = ConnMod:info(ConnProps),
     Sock = proplists:get_value(socket, Info),
     SockInfos = case rabbit_net:getstat(Sock,
             [recv_oct, recv_cnt, send_oct, send_cnt, send_pend]) of
